from __future__ import print_function
_A=None
import sys
from .compat import string_types
if False:from typing import Dict,Any,Text
class Node:
	__slots__='tag','value','start_mark','end_mark','comment','anchor'
	def __init__(A,tag,value,start_mark,end_mark,comment=_A,anchor=_A):A.tag=tag;A.value=value;A.start_mark=start_mark;A.end_mark=end_mark;A.comment=comment;A.anchor=anchor
	def __repr__(A):B=A.value;B=repr(B);return'%s(tag=%r, value=%s)'%(A.__class__.__name__,A.tag,B)
	def dump(A,indent=0):
		F='    {}comment: {})\n';D='  ';B=indent
		if isinstance(A.value,string_types):
			sys.stdout.write('{}{}(tag={!r}, value={!r})\n'.format(D*B,A.__class__.__name__,A.tag,A.value))
			if A.comment:sys.stdout.write(F.format(D*B,A.comment))
			return
		sys.stdout.write('{}{}(tag={!r})\n'.format(D*B,A.__class__.__name__,A.tag))
		if A.comment:sys.stdout.write(F.format(D*B,A.comment))
		for C in A.value:
			if isinstance(C,tuple):
				for E in C:E.dump(B+1)
			elif isinstance(C,Node):C.dump(B+1)
			else:sys.stdout.write('Node value type? {}\n'.format(type(C)))
class ScalarNode(Node):
	__slots__='style',;id='scalar'
	def __init__(A,tag,value,start_mark=_A,end_mark=_A,style=_A,comment=_A,anchor=_A):Node.__init__(A,tag,value,start_mark,end_mark,comment=comment,anchor=anchor);A.style=style
class CollectionNode(Node):
	__slots__='flow_style',
	def __init__(A,tag,value,start_mark=_A,end_mark=_A,flow_style=_A,comment=_A,anchor=_A):Node.__init__(A,tag,value,start_mark,end_mark,comment=comment);A.flow_style=flow_style;A.anchor=anchor
class SequenceNode(CollectionNode):__slots__=();id='sequence'
class MappingNode(CollectionNode):
	__slots__='merge',;id='mapping'
	def __init__(A,tag,value,start_mark=_A,end_mark=_A,flow_style=_A,comment=_A,anchor=_A):CollectionNode.__init__(A,tag,value,start_mark,end_mark,flow_style,comment,anchor);A.merge=_A


from __future__ import absolute_import
_A=None
from _ruamel_yaml import CParser,CEmitter
from .constructor import Constructor,BaseConstructor,SafeConstructor
from .representer import Representer,SafeRepresenter,BaseRepresenter
from .resolver import Resolver,BaseResolver
if False:from typing import Any,Union,Optional;from .compat import StreamTextType,StreamType,VersionType
__all__=['CBaseLoader','CSafeLoader','CLoader','CBaseDumper','CSafeDumper','CDumper']
class CBaseLoader(CParser,BaseConstructor,BaseResolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):CParser.__init__(A,stream);A._parser=A._composer=A;BaseConstructor.__init__(A,loader=A);BaseResolver.__init__(A,loadumper=A)
class CSafeLoader(CParser,SafeConstructor,Resolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):CParser.__init__(A,stream);A._parser=A._composer=A;SafeConstructor.__init__(A,loader=A);Resolver.__init__(A,loadumper=A)
class CLoader(CParser,Constructor,Resolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):CParser.__init__(A,stream);A._parser=A._composer=A;Constructor.__init__(A,loader=A);Resolver.__init__(A,loadumper=A)
class CBaseDumper(CEmitter,BaseRepresenter,BaseResolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):CEmitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,encoding=encoding,allow_unicode=allow_unicode,line_break=line_break,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags);A._emitter=A._serializer=A._representer=A;BaseRepresenter.__init__(A,default_style=default_style,default_flow_style=default_flow_style,dumper=A);BaseResolver.__init__(A,loadumper=A)
class CSafeDumper(CEmitter,SafeRepresenter,Resolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):A._emitter=A._serializer=A._representer=A;CEmitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,encoding=encoding,allow_unicode=allow_unicode,line_break=line_break,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags);A._emitter=A._serializer=A._representer=A;SafeRepresenter.__init__(A,default_style=default_style,default_flow_style=default_flow_style);Resolver.__init__(A)
class CDumper(CEmitter,Representer,Resolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):CEmitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,encoding=encoding,allow_unicode=allow_unicode,line_break=line_break,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags);A._emitter=A._serializer=A._representer=A;Representer.__init__(A,default_style=default_style,default_flow_style=default_flow_style);Resolver.__init__(A)


_A=None
from .compat import IS_TYPE_CHECKING
from .main import load_dotenv,get_key,set_key,unset_key,find_dotenv,dotenv_values
if IS_TYPE_CHECKING:from typing import Any,Optional
def load_ipython_extension(ipython):from .ipython import load_ipython_extension as A;A(ipython)
def get_cli_string(path=_A,action=_A,key=_A,value=_A,quote=_A):
	E=' ';D=quote;C=action;B=value;A=['dotenv']
	if D:A.append('-q %s'%D)
	if path:A.append('-f %s'%path)
	if C:
		A.append(C)
		if key:
			A.append(key)
			if B:
				if E in B:A.append('"%s"'%B)
				else:A.append(B)
	return E.join(A).strip()
__all__=['get_cli_string','load_dotenv','dotenv_values','get_key','set_key','unset_key','find_dotenv','load_ipython_extension']


from __future__ import print_function,absolute_import,division,unicode_literals
_V='bdist_wheel'
_U='--version'
_T='extra_packages'
_S='universal'
_R='nested'
_Q='setting  distdir {}/{}'
_P='nsp'
_O='PYDISTBASE'
_N='True'
_M='DVDEBUG'
_L='LICENSE'
_K='Jython'
_J='install'
_I='full_package_name'
_H='__init__.py'
_G='python'
_F='setup.py'
_E='utf-8'
_D=True
_C=False
_B='.'
_A=None
import sys,os,datetime,traceback
sys.path=[path for path in sys.path if path not in[os.getcwd(),'']]
import platform
from _ast import *
from ast import parse
from setuptools import setup,Extension,Distribution
from setuptools.command import install_lib
from setuptools.command.sdist import sdist as _sdist
try:from setuptools.namespaces import Installer as NameSpaceInstaller
except ImportError:msg='You should use the latest setuptools. The namespaces.py file that this setup.py uses was added in setuptools 28.7.0 (Oct 2016)';print(msg);sys.exit()
if __name__!='__main__':raise NotImplementedError('should never include setup.py')
full_package_name=_A
if sys.version_info<(3,):string_type=basestring
else:string_type=str
if sys.version_info<(3,4):
	class Bytes:0
	class NameConstant:0
if sys.version_info>=(3,8):from ast import Str,Num,Bytes,NameConstant
if sys.version_info<(3,):open_kw=dict()
else:open_kw=dict(encoding=_E)
if sys.version_info<(2,7)or platform.python_implementation()==_K:
	class Set:0
if os.environ.get(_M,'')=='':
	def debug(*args,**kw):0
else:
	def debug(*args,**kw):
		with open(os.environ[_M],'a')as fp:kw1=kw.copy();kw1['file']=fp;print('{:%Y-%d-%mT%H:%M:%S}'.format(datetime.datetime.now()),file=fp,end=' ');print(*args,**kw1)
def literal_eval(node_or_string):
	_safe_names={'None':_A,_N:_D,'False':_C}
	if isinstance(node_or_string,string_type):node_or_string=parse(node_or_string,mode='eval')
	if isinstance(node_or_string,Expression):node_or_string=node_or_string.body
	else:raise TypeError('only string or AST nodes supported')
	def _convert(node):
		if isinstance(node,Str):
			if sys.version_info<(3,)and not isinstance(node.s,unicode):return node.s.decode(_E)
			return node.s
		elif isinstance(node,Bytes):return node.s
		elif isinstance(node,Num):return node.n
		elif isinstance(node,Tuple):return tuple(map(_convert,node.elts))
		elif isinstance(node,List):return list(map(_convert,node.elts))
		elif isinstance(node,Set):return set(map(_convert,node.elts))
		elif isinstance(node,Dict):return dict(((_convert(k),_convert(v))for(k,v)in zip(node.keys,node.values)))
		elif isinstance(node,NameConstant):return node.value
		elif sys.version_info<(3,4)and isinstance(node,Name):
			if node.id in _safe_names:return _safe_names[node.id]
		elif isinstance(node,UnaryOp)and isinstance(node.op,(UAdd,USub))and isinstance(node.operand,(Num,UnaryOp,BinOp)):
			operand=_convert(node.operand)
			if isinstance(node.op,UAdd):return+operand
			else:return-operand
		elif isinstance(node,BinOp)and isinstance(node.op,(Add,Sub))and isinstance(node.right,(Num,UnaryOp,BinOp))and isinstance(node.left,(Num,UnaryOp,BinOp)):
			left=_convert(node.left);right=_convert(node.right)
			if isinstance(node.op,Add):return left+right
			else:return left-right
		elif isinstance(node,Call):
			func_id=getattr(node.func,'id',_A)
			if func_id=='dict':return dict(((k.arg,_convert(k.value))for k in node.keywords))
			elif func_id=='set':return set(_convert(node.args[0]))
			elif func_id=='date':return datetime.date(*[_convert(k)for k in node.args])
			elif func_id=='datetime':return datetime.datetime(*[_convert(k)for k in node.args])
		err=SyntaxError('malformed node or string: '+repr(node));err.filename='<string>';err.lineno=node.lineno;err.offset=node.col_offset;err.text=repr(node);err.node=node;raise err
	return _convert(node_or_string)
def _package_data(fn):
	data={}
	with open(fn,**open_kw)as fp:
		parsing=_C;lines=[]
		for line in fp.readlines():
			if sys.version_info<(3,):line=line.decode(_E)
			if line.startswith('_package_data'):
				if'dict('in line:parsing=_G;lines.append('dict(\n')
				elif line.endswith('= {\n'):parsing=_G;lines.append('{\n')
				else:raise NotImplementedError
				continue
			if not parsing:continue
			if parsing==_G:
				if line.startswith(')')or line.startswith('}'):
					lines.append(line)
					try:data=literal_eval(''.join(lines))
					except SyntaxError as e:
						context=2;from_line=e.lineno-(context+1);to_line=e.lineno+(context-1);w=len(str(to_line))
						for (index,line) in enumerate(lines):
							if from_line<=index<=to_line:
								print('{0:{1}}: {2}'.format(index,w,line).encode(_E),end='')
								if index==e.lineno-1:print('{0:{1}}  {2}^--- {3}'.format(' ',w,' '*e.offset,e.node))
						raise
					break
				lines.append(line)
			else:raise NotImplementedError
	return data
pkg_data=_package_data(__file__.replace(_F,_H))
exclude_files=[_F]
def _check_convert_version(tup):
	ret_val=str(tup[0]);next_sep=_B;nr_digits=0;post_dev=_C
	for x in tup[1:]:
		if isinstance(x,int):
			nr_digits+=1
			if nr_digits>2:raise ValueError('too many consecutive digits after '+ret_val)
			ret_val+=next_sep+str(x);next_sep=_B;continue
		first_letter=x[0].lower();next_sep=''
		if first_letter in'abcr':
			if post_dev:raise ValueError('release level specified after post/dev: '+x)
			nr_digits=0;ret_val+='rc'if first_letter=='r'else first_letter
		elif first_letter in'pd':nr_digits=1;post_dev=_D;ret_val+='.post'if first_letter=='p'else'.dev'
		else:raise ValueError('First letter of "'+x+'" not recognised')
	if nr_digits==1 and post_dev:ret_val+='0'
	return ret_val
version_info=pkg_data['version_info']
version_str=_check_convert_version(version_info)
class MyInstallLib(install_lib.install_lib):
	def install(self):
		fpp=pkg_data[_I].split(_B);full_exclude_files=[os.path.join(*fpp+[x])for x in exclude_files];alt_files=[];outfiles=install_lib.install_lib.install(self)
		for x in outfiles:
			for full_exclude_file in full_exclude_files:
				if full_exclude_file in x:os.remove(x);break
			else:alt_files.append(x)
		return alt_files
class MySdist(_sdist):
	def initialize_options(self):
		_sdist.initialize_options(self);dist_base=os.environ.get(_O);fpn=getattr(getattr(self,_P,self),_I,_A)
		if fpn and dist_base:print(_Q.format(dist_base,fpn));self.dist_dir=os.path.join(dist_base,fpn)
try:
	from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
	class MyBdistWheel(_bdist_wheel):
		def initialize_options(self):
			_bdist_wheel.initialize_options(self);dist_base=os.environ.get(_O);fpn=getattr(getattr(self,_P,self),_I,_A)
			if fpn and dist_base:print(_Q.format(dist_base,fpn));self.dist_dir=os.path.join(dist_base,fpn)
	_bdist_wheel_available=_D
except ImportError:_bdist_wheel_available=_C
class NameSpacePackager:
	def __init__(self,pkg_data):
		assert isinstance(pkg_data,dict);self._pkg_data=pkg_data;self.full_package_name=self.pn(self._pkg_data[_I]);self._split=_A;self.depth=self.full_package_name.count(_B);self.nested=self._pkg_data.get(_R,_C)
		if self.nested:NameSpaceInstaller.install_namespaces=lambda x:_A
		self.command=_A;self.python_version();self._pkg=[_A,_A]
		if sys.argv[0]==_F and sys.argv[1]==_J and'--single-version-externally-managed'not in sys.argv:
			if os.environ.get('READTHEDOCS',_A)==_N:os.system('pip install .');sys.exit(0)
			if not os.environ.get('RUAMEL_NO_PIP_INSTALL_CHECK',_C):print('error: you have to install with "pip install ."');sys.exit(1)
		if self._pkg_data.get(_S):Distribution.is_pure=lambda *args:_D
		else:Distribution.is_pure=lambda *args:_C
		for x in sys.argv:
			if x[0]=='-'or x==_F:continue
			self.command=x;break
	def pn(self,s):
		if sys.version_info<(3,)and isinstance(s,unicode):return s.encode(_E)
		return s
	@property
	def split(self):
		skip=[]
		if self._split is _A:
			fpn=self.full_package_name.split(_B);self._split=[]
			while fpn:self._split.insert(0,_B.join(fpn));fpn=fpn[:-1]
			for d in sorted(os.listdir(_B)):
				if not os.path.isdir(d)or d==self._split[0]or d[0]in'._':continue
				x=os.path.join(d,_H)
				if os.path.exists(x):
					pd=_package_data(x)
					if pd.get(_R,_C):skip.append(d);continue
					self._split.append(self.full_package_name+_B+d)
			if sys.version_info<(3,):self._split=[y.encode(_E)if isinstance(y,unicode)else y for y in self._split]
		if skip:0
		return self._split
	@property
	def namespace_packages(self):return self.split[:self.depth]
	def namespace_directories(self,depth=_A):
		res=[]
		for (index,d) in enumerate(self.split[:depth]):
			if index>0:d=os.path.join(*d.split(_B))
			res.append(_B+d)
		return res
	@property
	def package_dir(self):
		d={self.full_package_name:_B}
		if _T in self._pkg_data:return d
		if len(self.split)>1:d[self.split[0]]=self.namespace_directories(1)[0]
		return d
	def create_dirs(self):
		directories=self.namespace_directories(self.depth)
		if not directories:return
		if not os.path.exists(directories[0]):
			for d in directories:
				os.mkdir(d)
				with open(os.path.join(d,_H),'w')as fp:fp.write('import pkg_resources\npkg_resources.declare_namespace(__name__)\n')
	def python_version(self):
		supported=self._pkg_data.get('supported')
		if supported is _A:return
		if len(supported)==1:minimum=supported[0]
		else:
			for x in supported:
				if x[0]==sys.version_info[0]:minimum=x;break
			else:return
		if sys.version_info<minimum:print('minimum python version(s): '+str(supported));sys.exit(1)
	def check(self):
		A='develop'
		try:from pip.exceptions import InstallationError
		except ImportError:return
		if self.command not in[_J,A]:return
		prefix=self.split[0];prefixes=set([prefix,prefix.replace('_','-')])
		for p in sys.path:
			if not p:continue
			if os.path.exists(os.path.join(p,_F)):continue
			if not os.path.isdir(p):continue
			if p.startswith('/tmp/'):continue
			for fn in os.listdir(p):
				for pre in prefixes:
					if fn.startswith(pre):break
				else:continue
				full_name=os.path.join(p,fn)
				if fn==prefix and os.path.isdir(full_name):
					if self.command==A:raise InstallationError('Cannot mix develop (pip install -e),\nwith non-develop installs for package name {0}'.format(fn))
				elif fn==prefix:raise InstallationError('non directory package {0} in {1}'.format(fn,p))
				for pre in [x+_B for x in prefixes]:
					if fn.startswith(pre):break
				else:continue
				if fn.endswith('-link')and self.command==_J:raise InstallationError('Cannot mix non-develop with develop\n(pip install -e) installs for package name {0}'.format(fn))
	def entry_points(self,script_name=_A,package_name=_A):
		A='console_scripts'
		def pckg_entry_point(name):return '{0}{1}:main'.format(name,'.__main__'if os.path.exists('__main__.py')else'')
		ep=self._pkg_data.get('entry_points',_D)
		if isinstance(ep,dict):return ep
		if ep is _A:return _A
		if ep not in[_D,1]:
			if'='in ep:return{A:[ep]}
			script_name=ep
		if package_name is _A:package_name=self.full_package_name
		if not script_name:script_name=package_name.split(_B)[-1]
		return{A:['{0} = {1}'.format(script_name,pckg_entry_point(package_name))]}
	@property
	def url(self):
		url=self._pkg_data.get('url')
		if url:return url
		sp=self.full_package_name
		for ch in '_.':sp=sp.replace(ch,'-')
		return 'https://sourceforge.net/p/{0}/code/ci/default/tree'.format(sp)
	@property
	def author(self):return self._pkg_data['author']
	@property
	def author_email(self):return self._pkg_data['author_email']
	@property
	def license(self):
		lic=self._pkg_data.get('license')
		if lic is _A:return'MIT license'
		return lic
	def has_mit_lic(self):return'MIT'in self.license
	@property
	def description(self):return self._pkg_data['description']
	@property
	def status(self):
		A='β';status=self._pkg_data.get('status',A).lower()
		if status in['α','alpha']:return 3,'Alpha'
		elif status in[A,'beta']:return 4,'Beta'
		elif'stable'in status.lower():return 5,'Production/Stable'
		raise NotImplementedError
	@property
	def classifiers(self):
		attr='_'+sys._getframe().f_code.co_name
		if not hasattr(self,attr):setattr(self,attr,self._setup_classifiers())
		return getattr(self,attr)
	def _setup_classifiers(self):return sorted(set(['Development Status :: {0} - {1}'.format(*self.status),'Intended Audience :: Developers','License :: '+('OSI Approved :: MIT'if self.has_mit_lic()else'Other/Proprietary')+' License','Operating System :: OS Independent','Programming Language :: Python']+[self.pn(x)for x in self._pkg_data.get('classifiers',[])]))
	@property
	def keywords(self):return self.pn(self._pkg_data.get('keywords',[]))
	@property
	def install_requires(self):return self._analyse_packages[0]
	@property
	def install_pre(self):return self._analyse_packages[1]
	@property
	def _analyse_packages(self):
		if self._pkg[0]is _A:self._pkg[0]=[];self._pkg[1]=[]
		ir=self._pkg_data.get('install_requires')
		if ir is _A:return self._pkg
		if isinstance(ir,list):self._pkg[0]=ir;return self._pkg
		packages=ir.get('any',[])
		if isinstance(packages,string_type):packages=packages.split()
		if self.nested:
			parent_pkg=self.full_package_name.rsplit(_B,1)[0]
			if parent_pkg not in packages:packages.append(parent_pkg)
		implementation=platform.python_implementation()
		if implementation=='CPython':pyver='py{0}{1}'.format(*sys.version_info)
		elif implementation=='PyPy':pyver='pypy'if sys.version_info<(3,)else'pypy3'
		elif implementation==_K:pyver='jython'
		packages.extend(ir.get(pyver,[]))
		for p in packages:
			if p[0]=='*':p=p[1:];self._pkg[1].append(p)
			self._pkg[0].append(p)
		return self._pkg
	@property
	def extras_require(self):ep=self._pkg_data.get('extras_require');return ep
	@property
	def package_data(self):
		df=self._pkg_data.get('data_files',[])
		if self.has_mit_lic():df.append(_L);exclude_files.append(_L)
		if self._pkg_data.get('binary_only',_C):exclude_files.append(_H)
		debug('testing<<<<<')
		if'Typing :: Typed'in self.classifiers:debug('appending');df.append('py.typed')
		pd=self._pkg_data.get('package_data',{})
		if df:pd[self.full_package_name]=df
		if sys.version_info<(3,):
			for k in pd:
				if isinstance(k,unicode):pd[str(k)]=pd.pop(k)
		return pd
	@property
	def packages(self):s=self.split;return s+self._pkg_data.get(_T,[])
	@property
	def python_requires(self):return self._pkg_data.get('python_requires',_A)
	@property
	def ext_modules(self):
		I='Exception:';H='link error';G='compile error:';F='Windows';E='lib';D='src';C='ext_modules';B='test';A='name'
		if hasattr(self,'_ext_modules'):return self._ext_modules
		if _U in sys.argv:return _A
		if platform.python_implementation()==_K:return _A
		try:
			plat=sys.argv.index('--plat-name')
			if'win'in sys.argv[plat+1]:return _A
		except ValueError:pass
		self._ext_modules=[];no_test_compile=_C
		if'--restructuredtext'in sys.argv:no_test_compile=_D
		elif'sdist'in sys.argv:no_test_compile=_D
		if no_test_compile:
			for target in self._pkg_data.get(C,[]):ext=Extension(self.pn(target[A]),sources=[self.pn(x)for x in target[D]],libraries=[self.pn(x)for x in target.get(E)]);self._ext_modules.append(ext)
			return self._ext_modules
		print('sys.argv',sys.argv);import tempfile,shutil;from textwrap import dedent;import distutils.sysconfig,distutils.ccompiler;from distutils.errors import CompileError,LinkError
		for target in self._pkg_data.get(C,[]):
			ext=Extension(self.pn(target[A]),sources=[self.pn(x)for x in target[D]],libraries=[self.pn(x)for x in target.get(E)])
			if B not in target:self._ext_modules.append(ext);continue
			if sys.version_info[:2]==(3,4)and platform.system()==F:
				if'FORCE_C_BUILD_TEST'not in os.environ:self._ext_modules.append(ext);continue
			c_code=dedent(target[B])
			try:
				tmp_dir=tempfile.mkdtemp(prefix='tmp_ruamel_');bin_file_name=B+self.pn(target[A]);print('test compiling',bin_file_name);file_name=os.path.join(tmp_dir,bin_file_name+'.c')
				with open(file_name,'w')as fp:fp.write(c_code)
				compiler=distutils.ccompiler.new_compiler();assert isinstance(compiler,distutils.ccompiler.CCompiler);distutils.sysconfig.customize_compiler(compiler);compiler.add_include_dir(os.getcwd())
				if sys.version_info<(3,):tmp_dir=tmp_dir.encode(_E)
				compile_out_dir=tmp_dir
				try:compiler.link_executable(compiler.compile([file_name],output_dir=compile_out_dir),bin_file_name,output_dir=tmp_dir,libraries=ext.libraries)
				except CompileError:debug(G,file_name);print(G,file_name);continue
				except LinkError:debug(H,file_name);print(H,file_name);continue
				self._ext_modules.append(ext)
			except Exception as e:
				debug(I,e);print(I,e)
				if sys.version_info[:2]==(3,4)and platform.system()==F:traceback.print_exc()
			finally:shutil.rmtree(tmp_dir)
		return self._ext_modules
	@property
	def test_suite(self):return self._pkg_data.get('test_suite')
	def wheel(self,kw,setup):
		if _V not in sys.argv:return _C
		file_name='setup.cfg'
		if os.path.exists(file_name):return _C
		with open(file_name,'w')as fp:
			if os.path.exists(_L):fp.write('[metadata]\nlicense-file = LICENSE\n')
			else:print('\n\n>>>>>> LICENSE file not found <<<<<\n\n')
			if self._pkg_data.get(_S):fp.write('[bdist_wheel]\nuniversal = 1\n')
		try:setup(**kw)
		except Exception:raise
		finally:os.remove(file_name)
		return _D
def main():
	A='tarfmt';dump_kw='--dump-kw'
	if dump_kw in sys.argv:import wheel,distutils,setuptools;print('python:    ',sys.version);print('setuptools:',setuptools.__version__);print('distutils: ',distutils.__version__);print('wheel:     ',wheel.__version__)
	nsp=NameSpacePackager(pkg_data);nsp.check();nsp.create_dirs();MySdist.nsp=nsp
	if pkg_data.get(A):MySdist.tarfmt=pkg_data.get(A)
	cmdclass=dict(install_lib=MyInstallLib,sdist=MySdist)
	if _bdist_wheel_available:MyBdistWheel.nsp=nsp;cmdclass[_V]=MyBdistWheel
	kw=dict(name=nsp.full_package_name,namespace_packages=nsp.namespace_packages,version=version_str,packages=nsp.packages,python_requires=nsp.python_requires,url=nsp.url,author=nsp.author,author_email=nsp.author_email,cmdclass=cmdclass,package_dir=nsp.package_dir,entry_points=nsp.entry_points(),description=nsp.description,install_requires=nsp.install_requires,extras_require=nsp.extras_require,license=nsp.license,classifiers=nsp.classifiers,keywords=nsp.keywords,package_data=nsp.package_data,ext_modules=nsp.ext_modules,test_suite=nsp.test_suite)
	if _U not in sys.argv and('--verbose'in sys.argv or dump_kw in sys.argv):
		for k in sorted(kw):v=kw[k];print('  "{0}": "{1}",'.format(k,v))
	if dump_kw in sys.argv:sys.argv.remove(dump_kw)
	try:
		with open('README.rst')as fp:kw['long_description']=fp.read();kw['long_description_content_type']='text/x-rst'
	except Exception:pass
	if nsp.wheel(kw,setup):return
	for x in ['-c','egg_info','--egg-base','pip-egg-info']:
		if x not in sys.argv:break
	else:
		for p in nsp.install_pre:
			import subprocess;setup_path=os.path.join(*p.split(_B)+[_F]);try_dir=os.path.dirname(sys.executable)
			while len(try_dir)>1:
				full_path_setup_py=os.path.join(try_dir,setup_path)
				if os.path.exists(full_path_setup_py):pip=sys.executable.replace(_G,'pip');cmd=[pip,_J,os.path.dirname(full_path_setup_py)];subprocess.check_output(cmd);break
				try_dir=os.path.dirname(try_dir)
	setup(**kw)
main()


from __future__ import absolute_import,print_function
_G='_od'
_F='CommentedMap'
_E='# '
_D=True
_C='\n'
_B=False
_A=None
import sys,copy
from .compat import ordereddict
from .compat import PY2,string_types,MutableSliceableSequence
from .scalarstring import ScalarString
from .anchor import Anchor
if PY2:from collections import MutableSet,Sized,Set,Mapping
else:from collections.abc import MutableSet,Sized,Set,Mapping
if _B:from typing import Any,Dict,Optional,List,Union,Optional,Iterator
__all__=['CommentedSeq','CommentedKeySeq',_F,'CommentedOrderedMap','CommentedSet','comment_attrib','merge_attrib']
comment_attrib='_yaml_comment'
format_attrib='_yaml_format'
line_col_attrib='_yaml_line_col'
merge_attrib='_yaml_merge'
tag_attrib='_yaml_tag'
class Comment:
	__slots__='comment','_items','_end','_start';attrib=comment_attrib
	def __init__(A):A.comment=_A;A._items={};A._end=[]
	def __str__(A):
		if bool(A._end):B=',\n  end='+str(A._end)
		else:B=''
		return 'Comment(comment={0},\n  items={1}{2})'.format(A.comment,A._items,B)
	@property
	def items(self):return self._items
	@property
	def end(self):return self._end
	@end.setter
	def end(self,value):self._end=value
	@property
	def start(self):return self._start
	@start.setter
	def start(self,value):self._start=value
def NoComment():0
class Format:
	__slots__='_flow_style',;attrib=format_attrib
	def __init__(A):A._flow_style=_A
	def set_flow_style(A):A._flow_style=_D
	def set_block_style(A):A._flow_style=_B
	def flow_style(A,default=_A):
		if A._flow_style is _A:return default
		return A._flow_style
class LineCol:
	attrib=line_col_attrib
	def __init__(A):A.line=_A;A.col=_A;A.data=_A
	def add_kv_line_col(A,key,data):
		if A.data is _A:A.data={}
		A.data[key]=data
	def key(A,k):return A._kv(k,0,1)
	def value(A,k):return A._kv(k,2,3)
	def _kv(A,k,x0,x1):
		if A.data is _A:return _A
		B=A.data[k];return B[x0],B[x1]
	def item(A,idx):
		if A.data is _A:return _A
		return A.data[idx][0],A.data[idx][1]
	def add_idx_line_col(A,key,data):
		if A.data is _A:A.data={}
		A.data[key]=data
class Tag:
	__slots__='value',;attrib=tag_attrib
	def __init__(A):A.value=_A
	def __repr__(A):return '{0.__class__.__name__}({0.value!r})'.format(A)
class CommentedBase:
	@property
	def ca(self):
		A=self
		if not hasattr(A,Comment.attrib):setattr(A,Comment.attrib,Comment())
		return getattr(A,Comment.attrib)
	def yaml_end_comment_extend(A,comment,clear=_B):
		B=comment
		if B is _A:return
		if clear or A.ca.end is _A:A.ca.end=[]
		A.ca.end.extend(B)
	def yaml_key_comment_extend(C,key,comment,clear=_B):
		A=comment;B=C.ca._items.setdefault(key,[_A,_A,_A,_A])
		if clear or B[1]is _A:
			if A[1]is not _A:assert isinstance(A[1],list)
			B[1]=A[1]
		else:B[1].extend(A[0])
		B[0]=A[0]
	def yaml_value_comment_extend(C,key,comment,clear=_B):
		A=comment;B=C.ca._items.setdefault(key,[_A,_A,_A,_A])
		if clear or B[3]is _A:
			if A[1]is not _A:assert isinstance(A[1],list)
			B[3]=A[1]
		else:B[3].extend(A[0])
		B[2]=A[0]
	def yaml_set_start_comment(B,comment,indent=0):
		A=comment;from .error import CommentMark as C;from .tokens import CommentToken as D;E=B._yaml_get_pre_comment()
		if A[-1]==_C:A=A[:-1]
		F=C(indent)
		for G in A.split(_C):E.append(D(_E+G+_C,F,_A))
	def yaml_set_comment_before_after_key(J,key,before=_A,indent=0,after=_A,after_indent=_A):
		H=indent;E=after_indent;B=after;A=before;from dynaconf.vendor.ruamel.yaml.error import CommentMark as I;from dynaconf.vendor.ruamel.yaml.tokens import CommentToken as K
		def F(s,mark):return K((_E if s else'')+s+_C,mark,_A)
		if E is _A:E=H+2
		if A and len(A)>1 and A[-1]==_C:A=A[:-1]
		if B and B[-1]==_C:B=B[:-1]
		D=I(H);C=J.ca.items.setdefault(key,[_A,[],_A,_A])
		if A==_C:C[1].append(F('',D))
		elif A:
			for G in A.split(_C):C[1].append(F(G,D))
		if B:
			D=I(E)
			if C[3]is _A:C[3]=[]
			for G in B.split(_C):C[3].append(F(G,D))
	@property
	def fa(self):
		A=self
		if not hasattr(A,Format.attrib):setattr(A,Format.attrib,Format())
		return getattr(A,Format.attrib)
	def yaml_add_eol_comment(C,comment,key=NoComment,column=_A):
		H='#';B=column;A=comment;from .tokens import CommentToken as D;from .error import CommentMark as E
		if B is _A:
			try:B=C._yaml_get_column(key)
			except AttributeError:B=0
		if A[0]!=H:A=_E+A
		if B is _A:
			if A[0]==H:A=' '+A;B=0
		F=E(B);G=[D(A,F,_A),_A];C._yaml_add_eol_comment(G,key=key)
	@property
	def lc(self):
		A=self
		if not hasattr(A,LineCol.attrib):setattr(A,LineCol.attrib,LineCol())
		return getattr(A,LineCol.attrib)
	def _yaml_set_line_col(A,line,col):A.lc.line=line;A.lc.col=col
	def _yaml_set_kv_line_col(A,key,data):A.lc.add_kv_line_col(key,data)
	def _yaml_set_idx_line_col(A,key,data):A.lc.add_idx_line_col(key,data)
	@property
	def anchor(self):
		A=self
		if not hasattr(A,Anchor.attrib):setattr(A,Anchor.attrib,Anchor())
		return getattr(A,Anchor.attrib)
	def yaml_anchor(A):
		if not hasattr(A,Anchor.attrib):return _A
		return A.anchor
	def yaml_set_anchor(A,value,always_dump=_B):A.anchor.value=value;A.anchor.always_dump=always_dump
	@property
	def tag(self):
		A=self
		if not hasattr(A,Tag.attrib):setattr(A,Tag.attrib,Tag())
		return getattr(A,Tag.attrib)
	def yaml_set_tag(A,value):A.tag.value=value
	def copy_attributes(B,t,memo=_A):
		for A in [Comment.attrib,Format.attrib,LineCol.attrib,Anchor.attrib,Tag.attrib,merge_attrib]:
			if hasattr(B,A):
				if memo is not _A:setattr(t,A,copy.deepcopy(getattr(B,A,memo)))
				else:setattr(t,A,getattr(B,A))
	def _yaml_add_eol_comment(A,comment,key):raise NotImplementedError
	def _yaml_get_pre_comment(A):raise NotImplementedError
	def _yaml_get_column(A,key):raise NotImplementedError
class CommentedSeq(MutableSliceableSequence,list,CommentedBase):
	__slots__=Comment.attrib,'_lst'
	def __init__(A,*B,**C):list.__init__(A,*B,**C)
	def __getsingleitem__(A,idx):return list.__getitem__(A,idx)
	def __setsingleitem__(B,idx,value):
		C=idx;A=value
		if C<len(B):
			if isinstance(A,string_types)and not isinstance(A,ScalarString)and isinstance(B[C],ScalarString):A=type(B[C])(A)
		list.__setitem__(B,C,A)
	def __delsingleitem__(A,idx=_A):
		B=idx;list.__delitem__(A,B);A.ca.items.pop(B,_A)
		for C in sorted(A.ca.items):
			if C<B:continue
			A.ca.items[C-1]=A.ca.items.pop(C)
	def __len__(A):return list.__len__(A)
	def insert(A,idx,val):
		list.insert(A,idx,val)
		for B in sorted(A.ca.items,reverse=_D):
			if B<idx:break
			A.ca.items[B+1]=A.ca.items.pop(B)
	def extend(A,val):list.extend(A,val)
	def __eq__(A,other):return list.__eq__(A,other)
	def _yaml_add_comment(A,comment,key=NoComment):
		B=comment
		if key is not NoComment:A.yaml_key_comment_extend(key,B)
		else:A.ca.comment=B
	def _yaml_add_eol_comment(A,comment,key):A._yaml_add_comment(comment,key=key)
	def _yaml_get_columnX(A,key):return A.ca.items[key][0].start_mark.column
	def _yaml_get_column(A,key):
		C=key;E=_A;B=_A;F,G=C-1,C+1
		if F in A.ca.items:B=F
		elif G in A.ca.items:B=G
		else:
			for (D,H) in enumerate(A):
				if D>=C:break
				if D not in A.ca.items:continue
				B=D
		if B is not _A:E=A._yaml_get_columnX(B)
		return E
	def _yaml_get_pre_comment(A):
		B=[]
		if A.ca.comment is _A:A.ca.comment=[_A,B]
		else:A.ca.comment[1]=B
		return B
	def __deepcopy__(A,memo):
		C=memo;B=A.__class__();C[id(A)]=B
		for D in A:B.append(copy.deepcopy(D,C));A.copy_attributes(B,memo=C)
		return B
	def __add__(A,other):return list.__add__(A,other)
	def sort(A,key=_A,reverse=_B):
		C=reverse
		if key is _A:B=sorted(zip(A,range(len(A))),reverse=C);list.__init__(A,[A[0]for A in B])
		else:B=sorted(zip(map(key,list.__iter__(A)),range(len(A))),reverse=C);list.__init__(A,[list.__getitem__(A,C[1])for C in B])
		D=A.ca.items;A.ca._items={}
		for (F,G) in enumerate(B):
			E=G[1]
			if E in D:A.ca.items[F]=D[E]
	def __repr__(A):return list.__repr__(A)
class CommentedKeySeq(tuple,CommentedBase):
	def _yaml_add_comment(A,comment,key=NoComment):
		B=comment
		if key is not NoComment:A.yaml_key_comment_extend(key,B)
		else:A.ca.comment=B
	def _yaml_add_eol_comment(A,comment,key):A._yaml_add_comment(comment,key=key)
	def _yaml_get_columnX(A,key):return A.ca.items[key][0].start_mark.column
	def _yaml_get_column(A,key):
		C=key;E=_A;B=_A;F,G=C-1,C+1
		if F in A.ca.items:B=F
		elif G in A.ca.items:B=G
		else:
			for (D,H) in enumerate(A):
				if D>=C:break
				if D not in A.ca.items:continue
				B=D
		if B is not _A:E=A._yaml_get_columnX(B)
		return E
	def _yaml_get_pre_comment(A):
		B=[]
		if A.ca.comment is _A:A.ca.comment=[_A,B]
		else:A.ca.comment[1]=B
		return B
class CommentedMapView(Sized):
	__slots__='_mapping',
	def __init__(A,mapping):A._mapping=mapping
	def __len__(A):B=len(A._mapping);return B
class CommentedMapKeysView(CommentedMapView,Set):
	__slots__=()
	@classmethod
	def _from_iterable(A,it):return set(it)
	def __contains__(A,key):return key in A._mapping
	def __iter__(A):
		for B in A._mapping:yield B
class CommentedMapItemsView(CommentedMapView,Set):
	__slots__=()
	@classmethod
	def _from_iterable(A,it):return set(it)
	def __contains__(A,item):
		B,C=item
		try:D=A._mapping[B]
		except KeyError:return _B
		else:return D==C
	def __iter__(A):
		for B in A._mapping._keys():yield(B,A._mapping[B])
class CommentedMapValuesView(CommentedMapView):
	__slots__=()
	def __contains__(A,value):
		for B in A._mapping:
			if value==A._mapping[B]:return _D
		return _B
	def __iter__(A):
		for B in A._mapping._keys():yield A._mapping[B]
class CommentedMap(ordereddict,CommentedBase):
	__slots__=Comment.attrib,'_ok','_ref'
	def __init__(A,*B,**C):A._ok=set();A._ref=[];ordereddict.__init__(A,*B,**C)
	def _yaml_add_comment(A,comment,key=NoComment,value=NoComment):
		C=value;B=comment
		if key is not NoComment:A.yaml_key_comment_extend(key,B);return
		if C is not NoComment:A.yaml_value_comment_extend(C,B)
		else:A.ca.comment=B
	def _yaml_add_eol_comment(A,comment,key):A._yaml_add_comment(comment,value=key)
	def _yaml_get_columnX(A,key):return A.ca.items[key][2].start_mark.column
	def _yaml_get_column(A,key):
		E=key;H=_A;B=_A;C,F,I=_A,_A,_A
		for D in A:
			if C is not _A and D!=E:F=D;break
			if D==E:C=I
			I=D
		if C in A.ca.items:B=C
		elif F in A.ca.items:B=F
		else:
			for G in A:
				if G>=E:break
				if G not in A.ca.items:continue
				B=G
		if B is not _A:H=A._yaml_get_columnX(B)
		return H
	def _yaml_get_pre_comment(A):
		B=[]
		if A.ca.comment is _A:A.ca.comment=[_A,B]
		else:A.ca.comment[1]=B
		return B
	def update(B,vals):
		A=vals
		try:ordereddict.update(B,A)
		except TypeError:
			for C in A:B[C]=A[C]
		try:B._ok.update(A.keys())
		except AttributeError:
			for C in A:B._ok.add(C[0])
	def insert(A,pos,key,value,comment=_A):
		C=comment;B=key;ordereddict.insert(A,pos,B,value);A._ok.add(B)
		if C is not _A:A.yaml_add_eol_comment(C,key=B)
	def mlget(C,key,default=_A,list_ok=_B):
		D=list_ok;B=default;A=key
		if not isinstance(A,list):return C.get(A,B)
		def E(key_list,level,d):
			B=level;A=key_list
			if not D:assert isinstance(d,dict)
			if B>=len(A):
				if B>len(A):raise IndexError
				return d[A[B-1]]
			return E(A,B+1,d[A[B-1]])
		try:return E(A,1,C)
		except KeyError:return B
		except (TypeError,IndexError):
			if not D:raise
			return B
	def __getitem__(B,key):
		A=key
		try:return ordereddict.__getitem__(B,A)
		except KeyError:
			for C in getattr(B,merge_attrib,[]):
				if A in C[1]:return C[1][A]
			raise
	def __setitem__(A,key,value):
		C=value;B=key
		if B in A:
			if isinstance(C,string_types)and not isinstance(C,ScalarString)and isinstance(A[B],ScalarString):C=type(A[B])(C)
		ordereddict.__setitem__(A,B,C);A._ok.add(B)
	def _unmerged_contains(A,key):
		if key in A._ok:return _D
		return _A
	def __contains__(A,key):return bool(ordereddict.__contains__(A,key))
	def get(A,key,default=_A):
		try:return A.__getitem__(key)
		except:return default
	def __repr__(A):return ordereddict.__repr__(A).replace(_F,'ordereddict')
	def non_merged_items(A):
		for B in ordereddict.__iter__(A):
			if B in A._ok:yield(B,ordereddict.__getitem__(A,B))
	def __delitem__(A,key):
		B=key;A._ok.discard(B);ordereddict.__delitem__(A,B)
		for C in A._ref:C.update_key_value(B)
	def __iter__(A):
		for B in ordereddict.__iter__(A):yield B
	def _keys(A):
		for B in ordereddict.__iter__(A):yield B
	def __len__(A):return int(ordereddict.__len__(A))
	def __eq__(A,other):return bool(dict(A)==other)
	if PY2:
		def keys(A):return list(A._keys())
		def iterkeys(A):return A._keys()
		def viewkeys(A):return CommentedMapKeysView(A)
	else:
		def keys(A):return CommentedMapKeysView(A)
	if PY2:
		def _values(A):
			for B in ordereddict.__iter__(A):yield ordereddict.__getitem__(A,B)
		def values(A):return list(A._values())
		def itervalues(A):return A._values()
		def viewvalues(A):return CommentedMapValuesView(A)
	else:
		def values(A):return CommentedMapValuesView(A)
	def _items(A):
		for B in ordereddict.__iter__(A):yield(B,ordereddict.__getitem__(A,B))
	if PY2:
		def items(A):return list(A._items())
		def iteritems(A):return A._items()
		def viewitems(A):return CommentedMapItemsView(A)
	else:
		def items(A):return CommentedMapItemsView(A)
	@property
	def merge(self):
		A=self
		if not hasattr(A,merge_attrib):setattr(A,merge_attrib,[])
		return getattr(A,merge_attrib)
	def copy(A):
		B=type(A)()
		for (C,D) in A._items():B[C]=D
		A.copy_attributes(B);return B
	def add_referent(A,cm):
		if cm not in A._ref:A._ref.append(cm)
	def add_yaml_merge(A,value):
		C=value
		for B in C:
			B[1].add_referent(A)
			for (D,B) in B[1].items():
				if ordereddict.__contains__(A,D):continue
				ordereddict.__setitem__(A,D,B)
		A.merge.extend(C)
	def update_key_value(B,key):
		A=key
		if A in B._ok:return
		for C in B.merge:
			if A in C[1]:ordereddict.__setitem__(B,A,C[1][A]);return
		ordereddict.__delitem__(B,A)
	def __deepcopy__(A,memo):
		C=memo;B=A.__class__();C[id(A)]=B
		for D in A:B[D]=copy.deepcopy(A[D],C)
		A.copy_attributes(B,memo=C);return B
@classmethod
def raise_immutable(cls,*A,**B):raise TypeError('{} objects are immutable'.format(cls.__name__))
class CommentedKeyMap(CommentedBase,Mapping):
	__slots__=Comment.attrib,_G
	def __init__(A,*B,**C):
		if hasattr(A,_G):raise_immutable(A)
		try:A._od=ordereddict(*B,**C)
		except TypeError:
			if PY2:A._od=ordereddict(B[0].items())
			else:raise
	__delitem__=__setitem__=clear=pop=popitem=setdefault=update=raise_immutable
	def __getitem__(A,index):return A._od[index]
	def __iter__(A):
		for B in A._od.__iter__():yield B
	def __len__(A):return len(A._od)
	def __hash__(A):return hash(tuple(A.items()))
	def __repr__(A):
		if not hasattr(A,merge_attrib):return A._od.__repr__()
		return'ordereddict('+repr(list(A._od.items()))+')'
	@classmethod
	def fromkeys(A,v=_A):return CommentedKeyMap(dict.fromkeys(A,v))
	def _yaml_add_comment(A,comment,key=NoComment):
		B=comment
		if key is not NoComment:A.yaml_key_comment_extend(key,B)
		else:A.ca.comment=B
	def _yaml_add_eol_comment(A,comment,key):A._yaml_add_comment(comment,key=key)
	def _yaml_get_columnX(A,key):return A.ca.items[key][0].start_mark.column
	def _yaml_get_column(A,key):
		C=key;E=_A;B=_A;F,G=C-1,C+1
		if F in A.ca.items:B=F
		elif G in A.ca.items:B=G
		else:
			for (D,H) in enumerate(A):
				if D>=C:break
				if D not in A.ca.items:continue
				B=D
		if B is not _A:E=A._yaml_get_columnX(B)
		return E
	def _yaml_get_pre_comment(A):
		B=[]
		if A.ca.comment is _A:A.ca.comment=[_A,B]
		else:A.ca.comment[1]=B
		return B
class CommentedOrderedMap(CommentedMap):__slots__=Comment.attrib,
class CommentedSet(MutableSet,CommentedBase):
	__slots__=Comment.attrib,'odict'
	def __init__(A,values=_A):
		B=values;A.odict=ordereddict();MutableSet.__init__(A)
		if B is not _A:A|=B
	def _yaml_add_comment(A,comment,key=NoComment,value=NoComment):
		C=value;B=comment
		if key is not NoComment:A.yaml_key_comment_extend(key,B);return
		if C is not NoComment:A.yaml_value_comment_extend(C,B)
		else:A.ca.comment=B
	def _yaml_add_eol_comment(A,comment,key):A._yaml_add_comment(comment,value=key)
	def add(A,value):A.odict[value]=_A
	def discard(A,value):del A.odict[value]
	def __contains__(A,x):return x in A.odict
	def __iter__(A):
		for B in A.odict:yield B
	def __len__(A):return len(A.odict)
	def __repr__(A):return 'set({0!r})'.format(A.odict.keys())
class TaggedScalar(CommentedBase):
	def __init__(A,value=_A,style=_A,tag=_A):
		A.value=value;A.style=style
		if tag is not _A:A.yaml_set_tag(tag)
	def __str__(A):return A.value
def dump_comments(d,name='',sep='.',out=sys.stdout):
	G='ca';E='{}\n';D=out;C=sep;A=name
	if isinstance(d,dict)and hasattr(d,G):
		if A:sys.stdout.write(E.format(A))
		D.write(E.format(d.ca))
		for B in d:dump_comments(d[B],name=A+C+B if A else B,sep=C,out=D)
	elif isinstance(d,list)and hasattr(d,G):
		if A:sys.stdout.write(E.format(A))
		D.write(E.format(d.ca))
		for (F,B) in enumerate(d):dump_comments(B,name=A+C+str(F)if A else str(F),sep=C,out=D)


from __future__ import print_function,absolute_import,division,unicode_literals
_B=False
_A=None
import sys
from .compat import no_limit_int
from .anchor import Anchor
if _B:from typing import Text,Any,Dict,List
__all__=['ScalarFloat','ExponentialFloat','ExponentialCapsFloat']
class ScalarFloat(float):
	def __new__(D,*E,**A):
		F=A.pop('width',_A);G=A.pop('prec',_A);H=A.pop('m_sign',_A);I=A.pop('m_lead0',0);J=A.pop('exp',_A);K=A.pop('e_width',_A);L=A.pop('e_sign',_A);M=A.pop('underscore',_A);C=A.pop('anchor',_A);B=float.__new__(D,*E,**A);B._width=F;B._prec=G;B._m_sign=H;B._m_lead0=I;B._exp=J;B._e_width=K;B._e_sign=L;B._underscore=M
		if C is not _A:B.yaml_set_anchor(C,always_dump=True)
		return B
	def __iadd__(A,a):return float(A)+a;B=type(A)(A+a);B._width=A._width;B._underscore=A._underscore[:]if A._underscore is not _A else _A;return B
	def __ifloordiv__(A,a):return float(A)//a;B=type(A)(A//a);B._width=A._width;B._underscore=A._underscore[:]if A._underscore is not _A else _A;return B
	def __imul__(A,a):return float(A)*a;B=type(A)(A*a);B._width=A._width;B._underscore=A._underscore[:]if A._underscore is not _A else _A;B._prec=A._prec;return B
	def __ipow__(A,a):return float(A)**a;B=type(A)(A**a);B._width=A._width;B._underscore=A._underscore[:]if A._underscore is not _A else _A;return B
	def __isub__(A,a):return float(A)-a;B=type(A)(A-a);B._width=A._width;B._underscore=A._underscore[:]if A._underscore is not _A else _A;return B
	@property
	def anchor(self):
		A=self
		if not hasattr(A,Anchor.attrib):setattr(A,Anchor.attrib,Anchor())
		return getattr(A,Anchor.attrib)
	def yaml_anchor(A,any=_B):
		if not hasattr(A,Anchor.attrib):return _A
		if any or A.anchor.always_dump:return A.anchor
		return _A
	def yaml_set_anchor(A,value,always_dump=_B):A.anchor.value=value;A.anchor.always_dump=always_dump
	def dump(A,out=sys.stdout):out.write('ScalarFloat({}| w:{}, p:{}, s:{}, lz:{}, _:{}|{}, w:{}, s:{})\n'.format(A,A._width,A._prec,A._m_sign,A._m_lead0,A._underscore,A._exp,A._e_width,A._e_sign))
class ExponentialFloat(ScalarFloat):
	def __new__(A,value,width=_A,underscore=_A):return ScalarFloat.__new__(A,value,width=width,underscore=underscore)
class ExponentialCapsFloat(ScalarFloat):
	def __new__(A,value,width=_A,underscore=_A):return ScalarFloat.__new__(A,value,width=width,underscore=underscore)


from __future__ import print_function,absolute_import,division,unicode_literals
_A=False
import datetime,copy
if _A:from typing import Any,Dict,Optional,List
class TimeStamp(datetime.datetime):
	def __init__(A,*B,**C):A._yaml=dict(t=_A,tz=None,delta=0)
	def __new__(A,*B,**C):return datetime.datetime.__new__(A,*B,**C)
	def __deepcopy__(A,memo):B=TimeStamp(A.year,A.month,A.day,A.hour,A.minute,A.second);B._yaml=copy.deepcopy(A._yaml);return B


import os

from dynaconf import constants as ct
from dynaconf import default_settings
from dynaconf.loaders import ini_loader
from dynaconf.loaders import json_loader
from dynaconf.loaders import py_loader
from dynaconf.loaders import toml_loader
from dynaconf.loaders import yaml_loader
from dynaconf.utils import deduplicate
from dynaconf.utils import ensure_a_list
from dynaconf.utils.boxing import DynaBox
from dynaconf.utils.files import get_local_filename
from dynaconf.utils.parse_conf import false_values


def default_loader(obj, defaults=None):
    """Loads default settings and check if there are overridings
    exported as environment variables"""
    defaults = defaults or {}
    default_settings_values = {
        key: value
        for key, value in default_settings.__dict__.items()  # noqa
        if key.isupper()
    }

    all_keys = deduplicate(
        list(defaults.keys()) + list(default_settings_values.keys())
    )

    for key in all_keys:
        if not obj.exists(key):
            value = defaults.get(key, default_settings_values.get(key))
            obj.set(key, value)

    # start dotenv to get default env vars from there
    # check overrides in env vars
    if obj.get("load_dotenv") is True:
        default_settings.start_dotenv(obj)

    # Deal with cases where a custom ENV_SWITCHER_IS_PROVIDED
    # Example: Flask and Django Extensions
    env_switcher = defaults.get(
        "ENV_SWITCHER_FOR_DYNACONF", "ENV_FOR_DYNACONF"
    )

    for key in all_keys:
        if key not in default_settings_values.keys():
            continue

        env_value = obj.get_environ(
            env_switcher if key == "ENV_FOR_DYNACONF" else key,
            default="_not_found",
        )

        if env_value != "_not_found":
            obj.set(key, env_value, tomlfy=True)


def settings_loader(
    obj, settings_module=None, env=None, silent=True, key=None, filename=None
):
    """Loads from defined settings module

    :param obj: A dynaconf instance
    :param settings_module: A path or a list of paths e.g settings.toml
    :param env: Env to look for data defaults: development
    :param silent: Boolean to raise loading errors
    :param key: Load a single key if provided
    :param filename: optional filename to override the settings_module
    """
    if filename is None:
        settings_module = settings_module or obj.settings_module
        if not settings_module:  # pragma: no cover
            return
        files = ensure_a_list(settings_module)
    else:
        files = ensure_a_list(filename)

    files.extend(ensure_a_list(obj.get("SECRETS_FOR_DYNACONF", None)))

    found_files = []
    modules_names = []
    for item in files:
        item = str(item)  # Ensure str in case of LocalPath/Path is passed.
        if item.endswith(ct.ALL_EXTENSIONS + (".py",)):
            p_root = obj._root_path or (
                os.path.dirname(found_files[0]) if found_files else None
            )
            found = obj.find_file(item, project_root=p_root)
            if found:
                found_files.append(found)
        else:
            # a bare python module name w/o extension
            modules_names.append(item)

    enabled_core_loaders = obj.get("CORE_LOADERS_FOR_DYNACONF")

    # add `.local.` to found_files list to search for local files.
    found_files.extend(
        [
            get_local_filename(item)
            for item in found_files
            if ".local." not in str(item)
        ]
    )

    for mod_file in modules_names + found_files:
        # can be set to multiple files settings.py,settings.yaml,...

        # Cascade all loaders
        loaders = [
            {"ext": ct.YAML_EXTENSIONS, "name": "YAML", "loader": yaml_loader},
            {"ext": ct.TOML_EXTENSIONS, "name": "TOML", "loader": toml_loader},
            {"ext": ct.INI_EXTENSIONS, "name": "INI", "loader": ini_loader},
            {"ext": ct.JSON_EXTENSIONS, "name": "JSON", "loader": json_loader},
        ]

        for loader in loaders:
            if loader["name"] not in enabled_core_loaders:
                continue

            if mod_file.endswith(loader["ext"]):
                loader["loader"].load(
                    obj, filename=mod_file, env=env, silent=silent, key=key
                )
                continue

        if mod_file.endswith(ct.ALL_EXTENSIONS):
            continue

        if "PY" not in enabled_core_loaders:
            # pyloader is disabled
            continue

        # must be Python file or module
        # load from default defined module settings.py or .secrets.py if exists
        py_loader.load(obj, mod_file, key=key)

        # load from the current env e.g: development_settings.py
        env = env or obj.current_env
        if mod_file.endswith(".py"):
            if ".secrets.py" == mod_file:
                tmpl = ".{0}_{1}{2}"
                mod_file = "secrets.py"
            else:
                tmpl = "{0}_{1}{2}"

            dirname = os.path.dirname(mod_file)
            filename, extension = os.path.splitext(os.path.basename(mod_file))
            new_filename = tmpl.format(env.lower(), filename, extension)
            env_mod_file = os.path.join(dirname, new_filename)
            global_filename = tmpl.format("global", filename, extension)
            global_mod_file = os.path.join(dirname, global_filename)
        else:
            env_mod_file = f"{env.lower()}_{mod_file}"
            global_mod_file = f"global_{mod_file}"

        py_loader.load(
            obj,
            env_mod_file,
            identifier=f"py_{env.upper()}",
            silent=True,
            key=key,
        )

        # load from global_settings.py
        py_loader.load(
            obj, global_mod_file, identifier="py_global", silent=True, key=key
        )


def enable_external_loaders(obj):
    """Enable external service loaders like `VAULT_` and `REDIS_`
    looks forenv variables like `REDIS_ENABLED_FOR_DYNACONF`
    """
    for name, loader in ct.EXTERNAL_LOADERS.items():
        enabled = getattr(obj, f"{name.upper()}_ENABLED_FOR_DYNACONF", False)
        if (
            enabled
            and enabled not in false_values
            and loader not in obj.LOADERS_FOR_DYNACONF
        ):  # noqa
            obj.LOADERS_FOR_DYNACONF.insert(0, loader)


def write(filename, data, env=None):
    """Writes `data` to `filename` infers format by file extension."""
    loader_name = f"{filename.rpartition('.')[-1]}_loader"
    loader = globals().get(loader_name)
    if not loader:
        raise IOError(f"{loader_name} cannot be found.")

    data = DynaBox(data, box_settings={}).to_dict()
    if loader is not py_loader and env and env not in data:
        data = {env: data}

    loader.write(filename, data, merge=False)



from dynaconf.vendor.box.box import Box
class SBox(Box):
	_protected_keys=dir({})+['to_dict','to_json','to_yaml','json','yaml','from_yaml','from_json','dict','toml','from_toml','to_toml']
	@property
	def dict(self):return self.to_dict()
	@property
	def json(self):return self.to_json()
	@property
	def yaml(self):return self.to_yaml()
	@property
	def toml(self):return self.to_toml()
	def __repr__(A):return '<ShorthandBox: {0}>'.format(str(A.to_dict()))
	def copy(A):return SBox(super(SBox,A).copy())
	def __copy__(A):return SBox(super(SBox,A).copy())


from __future__ import absolute_import,unicode_literals,print_function
_Q='_emitter'
_P='_serializer'
_O='write'
_N='{}.dump(_all) takes two positional argument but at least three were given ({!r})'
_M='read'
_L='_stream'
_K='typ'
_J='utf-8'
_I='base'
_H='{}.__init__() takes no positional argument but at least one was given ({!r})'
_G='yaml_tag'
_F='open'
_E='rt'
_D='_'
_C=True
_B=False
_A=None
import sys,os,warnings,glob
from importlib import import_module
import dynaconf.vendor.ruamel as ruamel
from .error import UnsafeLoaderWarning,YAMLError
from .tokens import *
from .events import *
from .nodes import *
from .loader import BaseLoader,SafeLoader,Loader,RoundTripLoader
from .dumper import BaseDumper,SafeDumper,Dumper,RoundTripDumper
from .compat import StringIO,BytesIO,with_metaclass,PY3,nprint
from .resolver import VersionedResolver,Resolver
from .representer import BaseRepresenter,SafeRepresenter,Representer,RoundTripRepresenter
from .constructor import BaseConstructor,SafeConstructor,Constructor,RoundTripConstructor
from .loader import Loader as UnsafeLoader
if _B:
	from typing import List,Set,Dict,Union,Any,Callable,Optional,Text;from .compat import StreamType,StreamTextType,VersionType
	if PY3:from pathlib import Path
	else:Path=Any
try:from _ruamel_yaml import CParser,CEmitter
except:CParser=CEmitter=_A
enforce=object()
class YAML:
	def __init__(self,_kw=enforce,typ=_A,pure=_B,output=_A,plug_ins=_A):
		if _kw is not enforce:raise TypeError(_H.format(self.__class__.__name__,_kw))
		self.typ=[_E]if typ is _A else typ if isinstance(typ,list)else[typ];self.pure=pure;self._output=output;self._context_manager=_A;self.plug_ins=[]
		for pu in ([]if plug_ins is _A else plug_ins)+self.official_plug_ins():file_name=pu.replace(os.sep,'.');self.plug_ins.append(import_module(file_name))
		self.Resolver=ruamel.yaml.resolver.VersionedResolver;self.allow_unicode=_C;self.Reader=_A;self.Representer=_A;self.Constructor=_A;self.Scanner=_A;self.Serializer=_A;self.default_flow_style=_A;typ_found=1;setup_rt=_B
		if _E in self.typ:setup_rt=_C
		elif'safe'in self.typ:self.Emitter=ruamel.yaml.emitter.Emitter if pure or CEmitter is _A else CEmitter;self.Representer=ruamel.yaml.representer.SafeRepresenter;self.Parser=ruamel.yaml.parser.Parser if pure or CParser is _A else CParser;self.Composer=ruamel.yaml.composer.Composer;self.Constructor=ruamel.yaml.constructor.SafeConstructor
		elif _I in self.typ:self.Emitter=ruamel.yaml.emitter.Emitter;self.Representer=ruamel.yaml.representer.BaseRepresenter;self.Parser=ruamel.yaml.parser.Parser if pure or CParser is _A else CParser;self.Composer=ruamel.yaml.composer.Composer;self.Constructor=ruamel.yaml.constructor.BaseConstructor
		elif'unsafe'in self.typ:self.Emitter=ruamel.yaml.emitter.Emitter if pure or CEmitter is _A else CEmitter;self.Representer=ruamel.yaml.representer.Representer;self.Parser=ruamel.yaml.parser.Parser if pure or CParser is _A else CParser;self.Composer=ruamel.yaml.composer.Composer;self.Constructor=ruamel.yaml.constructor.Constructor
		else:setup_rt=_C;typ_found=0
		if setup_rt:self.default_flow_style=_B;self.Emitter=ruamel.yaml.emitter.Emitter;self.Serializer=ruamel.yaml.serializer.Serializer;self.Representer=ruamel.yaml.representer.RoundTripRepresenter;self.Scanner=ruamel.yaml.scanner.RoundTripScanner;self.Parser=ruamel.yaml.parser.RoundTripParser;self.Composer=ruamel.yaml.composer.Composer;self.Constructor=ruamel.yaml.constructor.RoundTripConstructor
		del setup_rt;self.stream=_A;self.canonical=_A;self.old_indent=_A;self.width=_A;self.line_break=_A;self.map_indent=_A;self.sequence_indent=_A;self.sequence_dash_offset=0;self.compact_seq_seq=_A;self.compact_seq_map=_A;self.sort_base_mapping_type_on_output=_A;self.top_level_colon_align=_A;self.prefix_colon=_A;self.version=_A;self.preserve_quotes=_A;self.allow_duplicate_keys=_B;self.encoding=_J;self.explicit_start=_A;self.explicit_end=_A;self.tags=_A;self.default_style=_A;self.top_level_block_style_scalar_no_indent_error_1_1=_B;self.scalar_after_indicator=_A;self.brace_single_entry_mapping_in_flow_sequence=_B
		for module in self.plug_ins:
			if getattr(module,_K,_A)in self.typ:typ_found+=1;module.init_typ(self);break
		if typ_found==0:raise NotImplementedError('typ "{}"not recognised (need to install plug-in?)'.format(self.typ))
	@property
	def reader(self):
		try:return self._reader
		except AttributeError:self._reader=self.Reader(_A,loader=self);return self._reader
	@property
	def scanner(self):
		try:return self._scanner
		except AttributeError:self._scanner=self.Scanner(loader=self);return self._scanner
	@property
	def parser(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):
			if self.Parser is not CParser:setattr(self,attr,self.Parser(loader=self))
			elif getattr(self,_L,_A)is _A:return _A
			else:setattr(self,attr,CParser(self._stream))
		return getattr(self,attr)
	@property
	def composer(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):setattr(self,attr,self.Composer(loader=self))
		return getattr(self,attr)
	@property
	def constructor(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):cnst=self.Constructor(preserve_quotes=self.preserve_quotes,loader=self);cnst.allow_duplicate_keys=self.allow_duplicate_keys;setattr(self,attr,cnst)
		return getattr(self,attr)
	@property
	def resolver(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):setattr(self,attr,self.Resolver(version=self.version,loader=self))
		return getattr(self,attr)
	@property
	def emitter(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):
			if self.Emitter is not CEmitter:
				_emitter=self.Emitter(_A,canonical=self.canonical,indent=self.old_indent,width=self.width,allow_unicode=self.allow_unicode,line_break=self.line_break,prefix_colon=self.prefix_colon,brace_single_entry_mapping_in_flow_sequence=self.brace_single_entry_mapping_in_flow_sequence,dumper=self);setattr(self,attr,_emitter)
				if self.map_indent is not _A:_emitter.best_map_indent=self.map_indent
				if self.sequence_indent is not _A:_emitter.best_sequence_indent=self.sequence_indent
				if self.sequence_dash_offset is not _A:_emitter.sequence_dash_offset=self.sequence_dash_offset
				if self.compact_seq_seq is not _A:_emitter.compact_seq_seq=self.compact_seq_seq
				if self.compact_seq_map is not _A:_emitter.compact_seq_map=self.compact_seq_map
			else:
				if getattr(self,_L,_A)is _A:return _A
				return _A
		return getattr(self,attr)
	@property
	def serializer(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):setattr(self,attr,self.Serializer(encoding=self.encoding,explicit_start=self.explicit_start,explicit_end=self.explicit_end,version=self.version,tags=self.tags,dumper=self))
		return getattr(self,attr)
	@property
	def representer(self):
		attr=_D+sys._getframe().f_code.co_name
		if not hasattr(self,attr):
			repres=self.Representer(default_style=self.default_style,default_flow_style=self.default_flow_style,dumper=self)
			if self.sort_base_mapping_type_on_output is not _A:repres.sort_base_mapping_type_on_output=self.sort_base_mapping_type_on_output
			setattr(self,attr,repres)
		return getattr(self,attr)
	def load(self,stream):
		if not hasattr(stream,_M)and hasattr(stream,_F):
			with stream.open('rb')as fp:return self.load(fp)
		constructor,parser=self.get_constructor_parser(stream)
		try:return constructor.get_single_data()
		finally:
			parser.dispose()
			try:self._reader.reset_reader()
			except AttributeError:pass
			try:self._scanner.reset_scanner()
			except AttributeError:pass
	def load_all(self,stream,_kw=enforce):
		if _kw is not enforce:raise TypeError(_H.format(self.__class__.__name__,_kw))
		if not hasattr(stream,_M)and hasattr(stream,_F):
			with stream.open('r')as fp:
				for d in self.load_all(fp,_kw=enforce):yield d
				return
		constructor,parser=self.get_constructor_parser(stream)
		try:
			while constructor.check_data():yield constructor.get_data()
		finally:
			parser.dispose()
			try:self._reader.reset_reader()
			except AttributeError:pass
			try:self._scanner.reset_scanner()
			except AttributeError:pass
	def get_constructor_parser(self,stream):
		if self.Parser is not CParser:
			if self.Reader is _A:self.Reader=ruamel.yaml.reader.Reader
			if self.Scanner is _A:self.Scanner=ruamel.yaml.scanner.Scanner
			self.reader.stream=stream
		elif self.Reader is not _A:
			if self.Scanner is _A:self.Scanner=ruamel.yaml.scanner.Scanner
			self.Parser=ruamel.yaml.parser.Parser;self.reader.stream=stream
		elif self.Scanner is not _A:
			if self.Reader is _A:self.Reader=ruamel.yaml.reader.Reader
			self.Parser=ruamel.yaml.parser.Parser;self.reader.stream=stream
		else:
			rslvr=self.Resolver
			class XLoader(self.Parser,self.Constructor,rslvr):
				def __init__(selfx,stream,version=self.version,preserve_quotes=_A):CParser.__init__(selfx,stream);selfx._parser=selfx._composer=selfx;self.Constructor.__init__(selfx,loader=selfx);selfx.allow_duplicate_keys=self.allow_duplicate_keys;rslvr.__init__(selfx,version=version,loadumper=selfx)
			self._stream=stream;loader=XLoader(stream);return loader,loader
		return self.constructor,self.parser
	def dump(self,data,stream=_A,_kw=enforce,transform=_A):
		if self._context_manager:
			if not self._output:raise TypeError('Missing output stream while dumping from context manager')
			if _kw is not enforce:raise TypeError('{}.dump() takes one positional argument but at least two were given ({!r})'.format(self.__class__.__name__,_kw))
			if transform is not _A:raise TypeError('{}.dump() in the context manager cannot have transform keyword '.format(self.__class__.__name__))
			self._context_manager.dump(data)
		else:
			if stream is _A:raise TypeError('Need a stream argument when not dumping from context manager')
			return self.dump_all([data],stream,_kw,transform=transform)
	def dump_all(self,documents,stream,_kw=enforce,transform=_A):
		if self._context_manager:raise NotImplementedError
		if _kw is not enforce:raise TypeError(_N.format(self.__class__.__name__,_kw))
		self._output=stream;self._context_manager=YAMLContextManager(self,transform=transform)
		for data in documents:self._context_manager.dump(data)
		self._context_manager.teardown_output();self._output=_A;self._context_manager=_A
	def Xdump_all(self,documents,stream,_kw=enforce,transform=_A):
		if not hasattr(stream,_O)and hasattr(stream,_F):
			with stream.open('w')as fp:return self.dump_all(documents,fp,_kw,transform=transform)
		if _kw is not enforce:raise TypeError(_N.format(self.__class__.__name__,_kw))
		if self.top_level_colon_align is _C:tlca=max([len(str(x))for x in documents[0]])
		else:tlca=self.top_level_colon_align
		if transform is not _A:
			fstream=stream
			if self.encoding is _A:stream=StringIO()
			else:stream=BytesIO()
		serializer,representer,emitter=self.get_serializer_representer_emitter(stream,tlca)
		try:
			self.serializer.open()
			for data in documents:
				try:self.representer.represent(data)
				except AttributeError:raise
			self.serializer.close()
		finally:
			try:self.emitter.dispose()
			except AttributeError:raise
			delattr(self,_P);delattr(self,_Q)
		if transform:
			val=stream.getvalue()
			if self.encoding:val=val.decode(self.encoding)
			if fstream is _A:transform(val)
			else:fstream.write(transform(val))
		return _A
	def get_serializer_representer_emitter(self,stream,tlca):
		if self.Emitter is not CEmitter:
			if self.Serializer is _A:self.Serializer=ruamel.yaml.serializer.Serializer
			self.emitter.stream=stream;self.emitter.top_level_colon_align=tlca
			if self.scalar_after_indicator is not _A:self.emitter.scalar_after_indicator=self.scalar_after_indicator
			return self.serializer,self.representer,self.emitter
		if self.Serializer is not _A:
			self.Emitter=ruamel.yaml.emitter.Emitter;self.emitter.stream=stream;self.emitter.top_level_colon_align=tlca
			if self.scalar_after_indicator is not _A:self.emitter.scalar_after_indicator=self.scalar_after_indicator
			return self.serializer,self.representer,self.emitter
		rslvr=ruamel.yaml.resolver.BaseResolver if _I in self.typ else ruamel.yaml.resolver.Resolver
		class XDumper(CEmitter,self.Representer,rslvr):
			def __init__(selfx,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):CEmitter.__init__(selfx,stream,canonical=canonical,indent=indent,width=width,encoding=encoding,allow_unicode=allow_unicode,line_break=line_break,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags);selfx._emitter=selfx._serializer=selfx._representer=selfx;self.Representer.__init__(selfx,default_style=default_style,default_flow_style=default_flow_style);rslvr.__init__(selfx)
		self._stream=stream;dumper=XDumper(stream,default_style=self.default_style,default_flow_style=self.default_flow_style,canonical=self.canonical,indent=self.old_indent,width=self.width,allow_unicode=self.allow_unicode,line_break=self.line_break,explicit_start=self.explicit_start,explicit_end=self.explicit_end,version=self.version,tags=self.tags);self._emitter=self._serializer=dumper;return dumper,dumper,dumper
	def map(self,**kw):
		if _E in self.typ:from dynaconf.vendor.ruamel.yaml.comments import CommentedMap;return CommentedMap(**kw)
		else:return dict(**kw)
	def seq(self,*args):
		if _E in self.typ:from dynaconf.vendor.ruamel.yaml.comments import CommentedSeq;return CommentedSeq(*args)
		else:return list(*args)
	def official_plug_ins(self):bd=os.path.dirname(__file__);gpbd=os.path.dirname(os.path.dirname(bd));res=[x.replace(gpbd,'')[1:-3]for x in glob.glob(bd+'/*/__plug_in__.py')];return res
	def register_class(self,cls):
		tag=getattr(cls,_G,'!'+cls.__name__)
		try:self.representer.add_representer(cls,cls.to_yaml)
		except AttributeError:
			def t_y(representer,data):return representer.represent_yaml_object(tag,data,cls,flow_style=representer.default_flow_style)
			self.representer.add_representer(cls,t_y)
		try:self.constructor.add_constructor(tag,cls.from_yaml)
		except AttributeError:
			def f_y(constructor,node):return constructor.construct_yaml_object(node,cls)
			self.constructor.add_constructor(tag,f_y)
		return cls
	def parse(self,stream):
		_,parser=self.get_constructor_parser(stream)
		try:
			while parser.check_event():yield parser.get_event()
		finally:
			parser.dispose()
			try:self._reader.reset_reader()
			except AttributeError:pass
			try:self._scanner.reset_scanner()
			except AttributeError:pass
	def __enter__(self):self._context_manager=YAMLContextManager(self);return self
	def __exit__(self,typ,value,traceback):
		if typ:nprint(_K,typ)
		self._context_manager.teardown_output();self._context_manager=_A
	def _indent(self,mapping=_A,sequence=_A,offset=_A):
		if mapping is not _A:self.map_indent=mapping
		if sequence is not _A:self.sequence_indent=sequence
		if offset is not _A:self.sequence_dash_offset=offset
	@property
	def indent(self):return self._indent
	@indent.setter
	def indent(self,val):self.old_indent=val
	@property
	def block_seq_indent(self):return self.sequence_dash_offset
	@block_seq_indent.setter
	def block_seq_indent(self,val):self.sequence_dash_offset=val
	def compact(self,seq_seq=_A,seq_map=_A):self.compact_seq_seq=seq_seq;self.compact_seq_map=seq_map
class YAMLContextManager:
	def __init__(self,yaml,transform=_A):
		self._yaml=yaml;self._output_inited=_B;self._output_path=_A;self._output=self._yaml._output;self._transform=transform
		if not hasattr(self._output,_O)and hasattr(self._output,_F):self._output_path=self._output;self._output=self._output_path.open('w')
		if self._transform is not _A:
			self._fstream=self._output
			if self._yaml.encoding is _A:self._output=StringIO()
			else:self._output=BytesIO()
	def teardown_output(self):
		if self._output_inited:self._yaml.serializer.close()
		else:return
		try:self._yaml.emitter.dispose()
		except AttributeError:raise
		try:delattr(self._yaml,_P);delattr(self._yaml,_Q)
		except AttributeError:raise
		if self._transform:
			val=self._output.getvalue()
			if self._yaml.encoding:val=val.decode(self._yaml.encoding)
			if self._fstream is _A:self._transform(val)
			else:self._fstream.write(self._transform(val));self._fstream.flush();self._output=self._fstream
		if self._output_path is not _A:self._output.close()
	def init_output(self,first_data):
		if self._yaml.top_level_colon_align is _C:tlca=max([len(str(x))for x in first_data])
		else:tlca=self._yaml.top_level_colon_align
		self._yaml.get_serializer_representer_emitter(self._output,tlca);self._yaml.serializer.open();self._output_inited=_C
	def dump(self,data):
		if not self._output_inited:self.init_output(data)
		try:self._yaml.representer.represent(data)
		except AttributeError:raise
def yaml_object(yml):
	def yo_deco(cls):
		tag=getattr(cls,_G,'!'+cls.__name__)
		try:yml.representer.add_representer(cls,cls.to_yaml)
		except AttributeError:
			def t_y(representer,data):return representer.represent_yaml_object(tag,data,cls,flow_style=representer.default_flow_style)
			yml.representer.add_representer(cls,t_y)
		try:yml.constructor.add_constructor(tag,cls.from_yaml)
		except AttributeError:
			def f_y(constructor,node):return constructor.construct_yaml_object(node,cls)
			yml.constructor.add_constructor(tag,f_y)
		return cls
	return yo_deco
def scan(stream,Loader=Loader):
	loader=Loader(stream)
	try:
		while loader.scanner.check_token():yield loader.scanner.get_token()
	finally:loader._parser.dispose()
def parse(stream,Loader=Loader):
	loader=Loader(stream)
	try:
		while loader._parser.check_event():yield loader._parser.get_event()
	finally:loader._parser.dispose()
def compose(stream,Loader=Loader):
	loader=Loader(stream)
	try:return loader.get_single_node()
	finally:loader.dispose()
def compose_all(stream,Loader=Loader):
	loader=Loader(stream)
	try:
		while loader.check_node():yield loader._composer.get_node()
	finally:loader._parser.dispose()
def load(stream,Loader=_A,version=_A,preserve_quotes=_A):
	if Loader is _A:warnings.warn(UnsafeLoaderWarning.text,UnsafeLoaderWarning,stacklevel=2);Loader=UnsafeLoader
	loader=Loader(stream,version,preserve_quotes=preserve_quotes)
	try:return loader._constructor.get_single_data()
	finally:
		loader._parser.dispose()
		try:loader._reader.reset_reader()
		except AttributeError:pass
		try:loader._scanner.reset_scanner()
		except AttributeError:pass
def load_all(stream,Loader=_A,version=_A,preserve_quotes=_A):
	if Loader is _A:warnings.warn(UnsafeLoaderWarning.text,UnsafeLoaderWarning,stacklevel=2);Loader=UnsafeLoader
	loader=Loader(stream,version,preserve_quotes=preserve_quotes)
	try:
		while loader._constructor.check_data():yield loader._constructor.get_data()
	finally:
		loader._parser.dispose()
		try:loader._reader.reset_reader()
		except AttributeError:pass
		try:loader._scanner.reset_scanner()
		except AttributeError:pass
def safe_load(stream,version=_A):return load(stream,SafeLoader,version)
def safe_load_all(stream,version=_A):return load_all(stream,SafeLoader,version)
def round_trip_load(stream,version=_A,preserve_quotes=_A):return load(stream,RoundTripLoader,version,preserve_quotes=preserve_quotes)
def round_trip_load_all(stream,version=_A,preserve_quotes=_A):return load_all(stream,RoundTripLoader,version,preserve_quotes=preserve_quotes)
def emit(events,stream=_A,Dumper=Dumper,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A):
	getvalue=_A
	if stream is _A:stream=StringIO();getvalue=stream.getvalue
	dumper=Dumper(stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break)
	try:
		for event in events:dumper.emit(event)
	finally:
		try:dumper._emitter.dispose()
		except AttributeError:raise;dumper.dispose()
	if getvalue is not _A:return getvalue()
enc=_A if PY3 else _J
def serialize_all(nodes,stream=_A,Dumper=Dumper,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=enc,explicit_start=_A,explicit_end=_A,version=_A,tags=_A):
	getvalue=_A
	if stream is _A:
		if encoding is _A:stream=StringIO()
		else:stream=BytesIO()
		getvalue=stream.getvalue
	dumper=Dumper(stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,encoding=encoding,version=version,tags=tags,explicit_start=explicit_start,explicit_end=explicit_end)
	try:
		dumper._serializer.open()
		for node in nodes:dumper.serialize(node)
		dumper._serializer.close()
	finally:
		try:dumper._emitter.dispose()
		except AttributeError:raise;dumper.dispose()
	if getvalue is not _A:return getvalue()
def serialize(node,stream=_A,Dumper=Dumper,**kwds):return serialize_all([node],stream,Dumper=Dumper,**kwds)
def dump_all(documents,stream=_A,Dumper=Dumper,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=enc,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):
	getvalue=_A
	if top_level_colon_align is _C:top_level_colon_align=max([len(str(x))for x in documents[0]])
	if stream is _A:
		if encoding is _A:stream=StringIO()
		else:stream=BytesIO()
		getvalue=stream.getvalue
	dumper=Dumper(stream,default_style=default_style,default_flow_style=default_flow_style,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,block_seq_indent=block_seq_indent,top_level_colon_align=top_level_colon_align,prefix_colon=prefix_colon)
	try:
		dumper._serializer.open()
		for data in documents:
			try:dumper._representer.represent(data)
			except AttributeError:raise
		dumper._serializer.close()
	finally:
		try:dumper._emitter.dispose()
		except AttributeError:raise;dumper.dispose()
	if getvalue is not _A:return getvalue()
	return _A
def dump(data,stream=_A,Dumper=Dumper,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=enc,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A):return dump_all([data],stream,Dumper=Dumper,default_style=default_style,default_flow_style=default_flow_style,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,block_seq_indent=block_seq_indent)
def safe_dump_all(documents,stream=_A,**kwds):return dump_all(documents,stream,Dumper=SafeDumper,**kwds)
def safe_dump(data,stream=_A,**kwds):return dump_all([data],stream,Dumper=SafeDumper,**kwds)
def round_trip_dump(data,stream=_A,Dumper=RoundTripDumper,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=enc,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):allow_unicode=_C if allow_unicode is _A else allow_unicode;return dump_all([data],stream,Dumper=Dumper,default_style=default_style,default_flow_style=default_flow_style,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,block_seq_indent=block_seq_indent,top_level_colon_align=top_level_colon_align,prefix_colon=prefix_colon)
def add_implicit_resolver(tag,regexp,first=_A,Loader=_A,Dumper=_A,resolver=Resolver):
	A='add_implicit_resolver'
	if Loader is _A and Dumper is _A:resolver.add_implicit_resolver(tag,regexp,first);return
	if Loader:
		if hasattr(Loader,A):Loader.add_implicit_resolver(tag,regexp,first)
		elif issubclass(Loader,(BaseLoader,SafeLoader,ruamel.yaml.loader.Loader,RoundTripLoader)):Resolver.add_implicit_resolver(tag,regexp,first)
		else:raise NotImplementedError
	if Dumper:
		if hasattr(Dumper,A):Dumper.add_implicit_resolver(tag,regexp,first)
		elif issubclass(Dumper,(BaseDumper,SafeDumper,ruamel.yaml.dumper.Dumper,RoundTripDumper)):Resolver.add_implicit_resolver(tag,regexp,first)
		else:raise NotImplementedError
def add_path_resolver(tag,path,kind=_A,Loader=_A,Dumper=_A,resolver=Resolver):
	A='add_path_resolver'
	if Loader is _A and Dumper is _A:resolver.add_path_resolver(tag,path,kind);return
	if Loader:
		if hasattr(Loader,A):Loader.add_path_resolver(tag,path,kind)
		elif issubclass(Loader,(BaseLoader,SafeLoader,ruamel.yaml.loader.Loader,RoundTripLoader)):Resolver.add_path_resolver(tag,path,kind)
		else:raise NotImplementedError
	if Dumper:
		if hasattr(Dumper,A):Dumper.add_path_resolver(tag,path,kind)
		elif issubclass(Dumper,(BaseDumper,SafeDumper,ruamel.yaml.dumper.Dumper,RoundTripDumper)):Resolver.add_path_resolver(tag,path,kind)
		else:raise NotImplementedError
def add_constructor(tag,object_constructor,Loader=_A,constructor=Constructor):
	if Loader is _A:constructor.add_constructor(tag,object_constructor)
	else:
		if hasattr(Loader,'add_constructor'):Loader.add_constructor(tag,object_constructor);return
		if issubclass(Loader,BaseLoader):BaseConstructor.add_constructor(tag,object_constructor)
		elif issubclass(Loader,SafeLoader):SafeConstructor.add_constructor(tag,object_constructor)
		elif issubclass(Loader,Loader):Constructor.add_constructor(tag,object_constructor)
		elif issubclass(Loader,RoundTripLoader):RoundTripConstructor.add_constructor(tag,object_constructor)
		else:raise NotImplementedError
def add_multi_constructor(tag_prefix,multi_constructor,Loader=_A,constructor=Constructor):
	if Loader is _A:constructor.add_multi_constructor(tag_prefix,multi_constructor)
	else:
		if _B and hasattr(Loader,'add_multi_constructor'):Loader.add_multi_constructor(tag_prefix,constructor);return
		if issubclass(Loader,BaseLoader):BaseConstructor.add_multi_constructor(tag_prefix,multi_constructor)
		elif issubclass(Loader,SafeLoader):SafeConstructor.add_multi_constructor(tag_prefix,multi_constructor)
		elif issubclass(Loader,ruamel.yaml.loader.Loader):Constructor.add_multi_constructor(tag_prefix,multi_constructor)
		elif issubclass(Loader,RoundTripLoader):RoundTripConstructor.add_multi_constructor(tag_prefix,multi_constructor)
		else:raise NotImplementedError
def add_representer(data_type,object_representer,Dumper=_A,representer=Representer):
	if Dumper is _A:representer.add_representer(data_type,object_representer)
	else:
		if hasattr(Dumper,'add_representer'):Dumper.add_representer(data_type,object_representer);return
		if issubclass(Dumper,BaseDumper):BaseRepresenter.add_representer(data_type,object_representer)
		elif issubclass(Dumper,SafeDumper):SafeRepresenter.add_representer(data_type,object_representer)
		elif issubclass(Dumper,Dumper):Representer.add_representer(data_type,object_representer)
		elif issubclass(Dumper,RoundTripDumper):RoundTripRepresenter.add_representer(data_type,object_representer)
		else:raise NotImplementedError
def add_multi_representer(data_type,multi_representer,Dumper=_A,representer=Representer):
	if Dumper is _A:representer.add_multi_representer(data_type,multi_representer)
	else:
		if hasattr(Dumper,'add_multi_representer'):Dumper.add_multi_representer(data_type,multi_representer);return
		if issubclass(Dumper,BaseDumper):BaseRepresenter.add_multi_representer(data_type,multi_representer)
		elif issubclass(Dumper,SafeDumper):SafeRepresenter.add_multi_representer(data_type,multi_representer)
		elif issubclass(Dumper,Dumper):Representer.add_multi_representer(data_type,multi_representer)
		elif issubclass(Dumper,RoundTripDumper):RoundTripRepresenter.add_multi_representer(data_type,multi_representer)
		else:raise NotImplementedError
class YAMLObjectMetaclass(type):
	def __init__(cls,name,bases,kwds):
		super(YAMLObjectMetaclass,cls).__init__(name,bases,kwds)
		if _G in kwds and kwds[_G]is not _A:cls.yaml_constructor.add_constructor(cls.yaml_tag,cls.from_yaml);cls.yaml_representer.add_representer(cls,cls.to_yaml)
class YAMLObject(with_metaclass(YAMLObjectMetaclass)):
	__slots__=();yaml_constructor=Constructor;yaml_representer=Representer;yaml_tag=_A;yaml_flow_style=_A
	@classmethod
	def from_yaml(cls,constructor,node):return constructor.construct_yaml_object(node,cls)
	@classmethod
	def to_yaml(cls,representer,data):return representer.represent_yaml_object(cls.yaml_tag,data,cls,flow_style=cls.yaml_flow_style)


import io
import json
from pathlib import Path

from dynaconf import default_settings
from dynaconf.constants import JSON_EXTENSIONS
from dynaconf.loaders.base import BaseLoader
from dynaconf.utils import object_merge
from dynaconf.utils.parse_conf import try_to_encode

try:  # pragma: no cover
    import commentjson
except ImportError:  # pragma: no cover
    commentjson = None


def load(obj, env=None, silent=True, key=None, filename=None):
    """
    Reads and loads in to "obj" a single key or all keys from source file.

    :param obj: the settings instance
    :param env: settings current env default='development'
    :param silent: if errors should raise
    :param key: if defined load a single key, else load all in env
    :param filename: Optional custom filename to load
    :return: None
    """
    if (
        obj.get("COMMENTJSON_ENABLED_FOR_DYNACONF") and commentjson
    ):  # pragma: no cover  # noqa
        file_reader = commentjson.load
        string_reader = commentjson.loads
    else:
        file_reader = json.load
        string_reader = json.loads

    loader = BaseLoader(
        obj=obj,
        env=env,
        identifier="json",
        extensions=JSON_EXTENSIONS,
        file_reader=file_reader,
        string_reader=string_reader,
    )
    loader.load(filename=filename, key=key, silent=silent)


def write(settings_path, settings_data, merge=True):
    """Write data to a settings file.

    :param settings_path: the filepath
    :param settings_data: a dictionary with data
    :param merge: boolean if existing file should be merged with new data
    """
    settings_path = Path(settings_path)
    if settings_path.exists() and merge:  # pragma: no cover
        with io.open(
            str(settings_path), encoding=default_settings.ENCODING_FOR_DYNACONF
        ) as open_file:
            object_merge(json.load(open_file), settings_data)

    with io.open(
        str(settings_path),
        "w",
        encoding=default_settings.ENCODING_FOR_DYNACONF,
    ) as open_file:
        json.dump(settings_data, open_file, cls=DynaconfEncoder)


class DynaconfEncoder(json.JSONEncoder):
    """Transform Dynaconf custom types instances to json representation"""

    def default(self, o):
        return try_to_encode(o, callback=super().default)



import textwrap
from contextlib import contextmanager
class TextWrapper(textwrap.TextWrapper):
	def _handle_long_word(E,reversed_chunks,cur_line,cur_len,width):
		B=cur_line;A=reversed_chunks;C=max(width-cur_len,1)
		if E.break_long_words:D=A[-1];F=D[:C];G=D[C:];B.append(F);A[-1]=G
		elif not B:B.append(A.pop())
	@contextmanager
	def extra_indent(self,indent):
		B=indent;A=self;C=A.initial_indent;D=A.subsequent_indent;A.initial_indent+=B;A.subsequent_indent+=B
		try:yield
		finally:A.initial_indent=C;A.subsequent_indent=D
	def indent_only(A,text):
		B=[]
		for (D,E) in enumerate(text.splitlines()):
			C=A.initial_indent
			if D>0:C=A.subsequent_indent
			B.append(f"{C}{E}")
		return '\n'.join(B)


import io
from pathlib import Path

from dynaconf import default_settings
from dynaconf.constants import INI_EXTENSIONS
from dynaconf.loaders.base import BaseLoader
from dynaconf.utils import object_merge

try:
    from configobj import ConfigObj
except ImportError:  # pragma: no cover
    ConfigObj = None


def load(obj, env=None, silent=True, key=None, filename=None):
    """
    Reads and loads in to "obj" a single key or all keys from source file.

    :param obj: the settings instance
    :param env: settings current env default='development'
    :param silent: if errors should raise
    :param key: if defined load a single key, else load all in env
    :param filename: Optional custom filename to load
    :return: None
    """
    if ConfigObj is None:  # pragma: no cover
        BaseLoader.warn_not_installed(obj, "ini")
        return

    loader = BaseLoader(
        obj=obj,
        env=env,
        identifier="ini",
        extensions=INI_EXTENSIONS,
        file_reader=lambda fileobj: ConfigObj(fileobj).dict(),
        string_reader=lambda strobj: ConfigObj(strobj.split("\n")).dict(),
    )
    loader.load(filename=filename, key=key, silent=silent)


def write(settings_path, settings_data, merge=True):
    """Write data to a settings file.

    :param settings_path: the filepath
    :param settings_data: a dictionary with data
    :param merge: boolean if existing file should be merged with new data
    """
    settings_path = Path(settings_path)
    if settings_path.exists() and merge:  # pragma: no cover
        with io.open(
            str(settings_path), encoding=default_settings.ENCODING_FOR_DYNACONF
        ) as open_file:
            object_merge(ConfigObj(open_file).dict(), settings_data)
    new = ConfigObj()
    new.update(settings_data)
    new.write(open(str(settings_path), "bw"))



import errno
import importlib
import inspect
import io
import types
from contextlib import suppress
from pathlib import Path

from dynaconf import default_settings
from dynaconf.utils import DynaconfDict
from dynaconf.utils import object_merge
from dynaconf.utils import upperfy
from dynaconf.utils.files import find_file


def load(obj, settings_module, identifier="py", silent=False, key=None):
    """Tries to import a python module"""
    mod, loaded_from = get_module(obj, settings_module, silent)
    if not (mod and loaded_from):
        return
    load_from_python_object(obj, mod, settings_module, key, identifier)


def load_from_python_object(
    obj, mod, settings_module, key=None, identifier=None
):
    file_merge = getattr(mod, "dynaconf_merge", False) or getattr(
        mod, "DYNACONF_MERGE", False
    )
    for setting in dir(mod):
        # at least 3 first chars should be upper to be considered a setting var
        if setting[:3].isupper():
            if key is None or key == setting:
                setting_value = getattr(mod, setting)
                obj.set(
                    setting,
                    setting_value,
                    loader_identifier=identifier,
                    merge=file_merge,
                )
    obj._loaded_files.append(mod.__file__)


def try_to_load_from_py_module_name(
    obj, name, key=None, identifier="py", silent=False
):
    """Try to load module by its string name.

    Arguments:
        obj {LAzySettings} -- Dynaconf settings instance
        name {str} -- Name of the module e.g: foo.bar.zaz

    Keyword Arguments:
        key {str} -- Single key to be loaded (default: {None})
        identifier {str} -- Name of identifier to store (default: 'py')
        silent {bool} -- Weather to raise or silence exceptions.
    """
    ctx = suppress(ImportError, TypeError) if silent else suppress()

    with ctx:
        mod = importlib.import_module(name)
        load_from_python_object(obj, mod, name, key, identifier)
        return True  # loaded ok!
    # if it reaches this point that means exception occurred, module not found.
    return False


def get_module(obj, filename, silent=False):
    try:
        mod = importlib.import_module(filename)
        loaded_from = "module"
    except (ImportError, TypeError):
        mod = import_from_filename(obj, filename, silent=silent)
        if mod and not mod._is_error:
            loaded_from = "filename"
        else:
            # it is important to return None in case of not loaded
            loaded_from = None
    return mod, loaded_from


def import_from_filename(obj, filename, silent=False):  # pragma: no cover
    """If settings_module is a filename path import it."""
    if filename in [item.filename for item in inspect.stack()]:
        raise ImportError(
            "Looks like you are loading dynaconf "
            f"from inside the {filename} file and then it is trying "
            "to load itself entering in a circular reference "
            "problem. To solve it you have to "
            "invoke your program from another root folder "
            "or rename your program file."
        )

    _find_file = getattr(obj, "find_file", find_file)
    if not filename.endswith(".py"):
        filename = f"{filename}.py"

    if filename in default_settings.SETTINGS_FILE_FOR_DYNACONF:
        silent = True
    mod = types.ModuleType(filename.rstrip(".py"))
    mod.__file__ = filename
    mod._is_error = False
    try:
        with io.open(
            _find_file(filename),
            encoding=default_settings.ENCODING_FOR_DYNACONF,
        ) as config_file:
            exec(compile(config_file.read(), filename, "exec"), mod.__dict__)
    except IOError as e:
        e.strerror = (
            f"py_loader: error loading file " f"({e.strerror} {filename})\n"
        )
        if silent and e.errno in (errno.ENOENT, errno.EISDIR):
            return
        mod._is_error = True
    return mod


def write(settings_path, settings_data, merge=True):
    """Write data to a settings file.

    :param settings_path: the filepath
    :param settings_data: a dictionary with data
    :param merge: boolean if existing file should be merged with new data
    """
    settings_path = Path(settings_path)
    if settings_path.exists() and merge:  # pragma: no cover
        existing = DynaconfDict()
        load(existing, str(settings_path))
        object_merge(existing, settings_data)
    with io.open(
        str(settings_path),
        "w",
        encoding=default_settings.ENCODING_FOR_DYNACONF,
    ) as f:
        f.writelines(
            [f"{upperfy(k)} = {repr(v)}\n" for k, v in settings_data.items()]
        )



_G='r'
_F='w'
_E=False
_D=True
_C='strict'
_B='utf-8'
_A=None
import csv,json,sys,warnings
from pathlib import Path
import dynaconf.vendor.ruamel.yaml as yaml
from dynaconf.vendor.box.exceptions import BoxError,BoxWarning
from dynaconf.vendor import toml
BOX_PARAMETERS='default_box','default_box_attr','conversion_box','frozen_box','camel_killer_box','box_safe_prefix','box_duplicates','ordered_box','default_box_none_transform','box_dots','modify_tuples_box','box_intact_types','box_recast'
def _exists(filename,create=_E):
	A=filename;B=Path(A)
	if create:
		try:B.touch(exist_ok=_D)
		except OSError as C:raise BoxError(f"Could not create file {A} - {C}")
		else:return
	if not B.exists():raise BoxError(f'File "{A}" does not exist')
	if not B.is_file():raise BoxError(f"{A} is not a file")
def _to_json(obj,filename=_A,encoding=_B,errors=_C,**C):
	A=filename;B=json.dumps(obj,ensure_ascii=_E,**C)
	if A:
		_exists(A,create=_D)
		with open(A,_F,encoding=encoding,errors=errors)as D:D.write(B if sys.version_info>=(3,0)else B.decode(_B))
	else:return B
def _from_json(json_string=_A,filename=_A,encoding=_B,errors=_C,multiline=_E,**B):
	D=json_string;A=filename
	if A:
		_exists(A)
		with open(A,_G,encoding=encoding,errors=errors)as E:
			if multiline:C=[json.loads(A.strip(),**B)for A in E if A.strip()and not A.strip().startswith('#')]
			else:C=json.load(E,**B)
	elif D:C=json.loads(D,**B)
	else:raise BoxError('from_json requires a string or filename')
	return C
def _to_yaml(obj,filename=_A,default_flow_style=_E,encoding=_B,errors=_C,**C):
	B=default_flow_style;A=filename
	if A:
		_exists(A,create=_D)
		with open(A,_F,encoding=encoding,errors=errors)as D:yaml.dump(obj,stream=D,default_flow_style=B,**C)
	else:return yaml.dump(obj,default_flow_style=B,**C)
def _from_yaml(yaml_string=_A,filename=_A,encoding=_B,errors=_C,**A):
	F='Loader';C=yaml_string;B=filename
	if F not in A:A[F]=yaml.SafeLoader
	if B:
		_exists(B)
		with open(B,_G,encoding=encoding,errors=errors)as E:D=yaml.load(E,**A)
	elif C:D=yaml.load(C,**A)
	else:raise BoxError('from_yaml requires a string or filename')
	return D
def _to_toml(obj,filename=_A,encoding=_B,errors=_C):
	A=filename
	if A:
		_exists(A,create=_D)
		with open(A,_F,encoding=encoding,errors=errors)as B:toml.dump(obj,B)
	else:return toml.dumps(obj)
def _from_toml(toml_string=_A,filename=_A,encoding=_B,errors=_C):
	B=toml_string;A=filename
	if A:
		_exists(A)
		with open(A,_G,encoding=encoding,errors=errors)as D:C=toml.load(D)
	elif B:C=toml.loads(B)
	else:raise BoxError('from_toml requires a string or filename')
	return C
def _to_csv(box_list,filename,encoding=_B,errors=_C):
	B=filename;A=box_list;C=list(A[0].keys())
	for E in A:
		if list(E.keys())!=C:raise BoxError('BoxList must contain the same dictionary structure for every item to convert to csv')
	if B:
		_exists(B,create=_D)
		with open(B,_F,encoding=encoding,errors=errors,newline='')as F:
			D=csv.DictWriter(F,fieldnames=C);D.writeheader()
			for G in A:D.writerow(G)
def _from_csv(filename,encoding=_B,errors=_C):
	A=filename;_exists(A)
	with open(A,_G,encoding=encoding,errors=errors,newline='')as B:C=csv.DictReader(B);return[A for A in C]


from os import environ

from dynaconf.utils import upperfy
from dynaconf.utils.parse_conf import parse_conf_data
from dynaconf.vendor.dotenv import cli as dotenv_cli


IDENTIFIER = "env"


def load(obj, env=None, silent=True, key=None):
    """Loads envvars with prefixes:

    `DYNACONF_` (default global) or `$(ENVVAR_PREFIX_FOR_DYNACONF)_`
    """
    global_prefix = obj.get("ENVVAR_PREFIX_FOR_DYNACONF")
    if global_prefix is False or global_prefix.upper() != "DYNACONF":
        load_from_env(obj, "DYNACONF", key, silent, IDENTIFIER + "_global")

    # Load the global env if exists and overwrite everything
    load_from_env(obj, global_prefix, key, silent, IDENTIFIER + "_global")


def load_from_env(
    obj,
    prefix=False,
    key=None,
    silent=False,
    identifier=IDENTIFIER,
    env=False,  # backwards compatibility bc renamed param
):
    if prefix is False and env is not False:
        prefix = env

    env_ = ""
    if prefix is not False:
        if not isinstance(prefix, str):
            raise TypeError("`prefix/env` must be str or False")

        prefix = prefix.upper()
        env_ = f"{prefix}_"

    if key:
        key = upperfy(key)
        value = environ.get(f"{env_}{key}")
        if value:
            try:  # obj is a Settings
                obj.set(key, value, loader_identifier=identifier, tomlfy=True)
            except AttributeError:  # obj is a dict
                obj[key] = parse_conf_data(
                    value, tomlfy=True, box_settings=obj
                )
    else:
        trim_len = len(env_)
        data = {
            key[trim_len:]: parse_conf_data(
                data, tomlfy=True, box_settings=obj
            )
            for key, data in environ.items()
            if key.startswith(env_)
        }
        if data:
            obj.update(data, loader_identifier=identifier)


def write(settings_path, settings_data, **kwargs):
    """Write data to .env file"""
    for key, value in settings_data.items():
        quote_mode = (
            isinstance(value, str)
            and (value.startswith("'") or value.startswith('"'))
        ) or isinstance(value, (list, dict))
        dotenv_cli.set_key(
            str(settings_path),
            key,
            str(value),
            quote_mode="always" if quote_mode else "none",
        )



from __future__ import absolute_import
_A=None
from .reader import Reader
from .scanner import Scanner,RoundTripScanner
from .parser import Parser,RoundTripParser
from .composer import Composer
from .constructor import BaseConstructor,SafeConstructor,Constructor,RoundTripConstructor
from .resolver import VersionedResolver
if False:from typing import Any,Dict,List,Union,Optional;from .compat import StreamTextType,VersionType
__all__=['BaseLoader','SafeLoader','Loader','RoundTripLoader']
class BaseLoader(Reader,Scanner,Parser,Composer,BaseConstructor,VersionedResolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):Reader.__init__(A,stream,loader=A);Scanner.__init__(A,loader=A);Parser.__init__(A,loader=A);Composer.__init__(A,loader=A);BaseConstructor.__init__(A,loader=A);VersionedResolver.__init__(A,version,loader=A)
class SafeLoader(Reader,Scanner,Parser,Composer,SafeConstructor,VersionedResolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):Reader.__init__(A,stream,loader=A);Scanner.__init__(A,loader=A);Parser.__init__(A,loader=A);Composer.__init__(A,loader=A);SafeConstructor.__init__(A,loader=A);VersionedResolver.__init__(A,version,loader=A)
class Loader(Reader,Scanner,Parser,Composer,Constructor,VersionedResolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):Reader.__init__(A,stream,loader=A);Scanner.__init__(A,loader=A);Parser.__init__(A,loader=A);Composer.__init__(A,loader=A);Constructor.__init__(A,loader=A);VersionedResolver.__init__(A,version,loader=A)
class RoundTripLoader(Reader,RoundTripScanner,RoundTripParser,Composer,RoundTripConstructor,VersionedResolver):
	def __init__(A,stream,version=_A,preserve_quotes=_A):Reader.__init__(A,stream,loader=A);RoundTripScanner.__init__(A,loader=A);RoundTripParser.__init__(A,loader=A);Composer.__init__(A,loader=A);RoundTripConstructor.__init__(A,preserve_quotes=preserve_quotes,loader=A);VersionedResolver.__init__(A,version,loader=A)


import warnings
from .util import configobj_walker as new_configobj_walker
if False:from typing import Any
def configobj_walker(cfg):warnings.warn('configobj_walker has moved to ruamel.yaml.util, please update your code');return new_configobj_walker(cfg)


from __future__ import absolute_import,print_function
_B='lazy_self'
_A=' '
from functools import partial
import re
from .compat import text_type,binary_type
if False:from typing import Any,Dict,Optional,List,Text;from .compat import StreamTextType
class LazyEval:
	def __init__(A,func,*C,**D):
		def B():B=func(*C,**D);object.__setattr__(A,_B,lambda:B);return B
		object.__setattr__(A,_B,B)
	def __getattribute__(B,name):
		A=object.__getattribute__(B,_B)
		if name==_B:return A
		return getattr(A(),name)
	def __setattr__(A,name,value):setattr(A.lazy_self(),name,value)
RegExp=partial(LazyEval,re.compile)
def load_yaml_guess_indent(stream,**N):
	D=stream;B=None;from .main import round_trip_load as O
	def K(l):
		A=0
		while A<len(l)and l[A]==_A:A+=1
		return A
	if isinstance(D,text_type):F=D
	elif isinstance(D,binary_type):F=D.decode('utf-8')
	else:F=D.read()
	G=B;H=B;L=B;E=B;I=0
	for C in F.splitlines():
		J=C.rstrip();P=J.lstrip()
		if P.startswith('- '):
			M=K(C);L=M-I;A=M+1
			while C[A]==_A:A+=1
			if C[A]=='#':continue
			H=A-I;break
		if G is B and E is not B and J:
			A=0
			while C[A]in' -':A+=1
			if A>E:G=A-E
		if J.endswith(':'):
			I=K(C);A=0
			while C[A]==_A:A+=1
			E=A;continue
		E=B
	if H is B and G is not B:H=G
	return O(F,**N),H,L
def configobj_walker(cfg):
	B=cfg;from configobj import ConfigObj as D;assert isinstance(B,D)
	for A in B.initial_comment:
		if A.strip():yield A
	for C in _walk_section(B):
		if C.strip():yield C
	for A in B.final_comment:
		if A.strip():yield A
def _walk_section(s,level=0):
	L='  ';I="'";H='\n';F=level;from configobj import Section as J;assert isinstance(s,J);D=L*F
	for A in s.scalars:
		for B in s.comments[A]:yield D+B.strip()
		C=s[A]
		if H in C:G=D+L;C='|\n'+G+C.strip().replace(H,H+G)
		elif':'in C:C=I+C.replace(I,"''")+I
		E='{0}{1}: {2}'.format(D,A,C);B=s.inline_comments[A]
		if B:E+=_A+B
		yield E
	for A in s.sections:
		for B in s.comments[A]:yield D+B.strip()
		E='{0}{1}:'.format(D,A);B=s.inline_comments[A]
		if B:E+=_A+B
		yield E
		for K in _walk_section(s[A],level=F+1):yield K


_W='Reserved escape sequence used'
_V='\\U'
_U='false'
_T='true'
_S='\t'
_R='}'
_Q='+'
_P='_'
_O='-'
_N=','
_M=']'
_L=' '
_K='{'
_J='='
_I='['
_H='\\'
_G='\n'
_F='.'
_E=None
_D="'"
_C='"'
_B=True
_A=False
import datetime,io
from os import linesep
import re,sys
from .tz import TomlTz
if sys.version_info<(3,):_range=xrange
else:unicode=str;_range=range;basestring=str;unichr=chr
def _detect_pathlib_path(p):
	if(3,4)<=sys.version_info:
		import pathlib as A
		if isinstance(p,A.PurePath):return _B
	return _A
def _ispath(p):
	if isinstance(p,(bytes,basestring)):return _B
	return _detect_pathlib_path(p)
def _getpath(p):
	if(3,6)<=sys.version_info:import os;return os.fspath(p)
	if _detect_pathlib_path(p):return str(p)
	return p
try:FNFError=FileNotFoundError
except NameError:FNFError=IOError
TIME_RE=re.compile('([0-9]{2}):([0-9]{2}):([0-9]{2})(\\.([0-9]{3,6}))?')
class TomlDecodeError(ValueError):
	def __init__(A,msg,doc,pos):C=doc;B=pos;D=C.count(_G,0,B)+1;E=B-C.rfind(_G,0,B);F='{} (line {} column {} char {})'.format(msg,D,E,B);ValueError.__init__(A,F);A.msg=msg;A.doc=C;A.pos=B;A.lineno=D;A.colno=E
_number_with_underscores=re.compile('([0-9])(_([0-9]))*')
class CommentValue:
	def __init__(A,val,comment,beginline,_dict):A.val=val;B=_G if beginline else _L;A.comment=B+comment;A._dict=_dict
	def __getitem__(A,key):return A.val[key]
	def __setitem__(A,key,value):A.val[key]=value
	def dump(A,dump_value_func):
		B=dump_value_func(A.val)
		if isinstance(A.val,A._dict):return A.comment+_G+unicode(B)
		else:return unicode(B)+A.comment
def _strictly_valid_num(n):
	n=n.strip()
	if not n:return _A
	if n[0]==_P:return _A
	if n[-1]==_P:return _A
	if'_.'in n or'._'in n:return _A
	if len(n)==1:return _B
	if n[0]=='0'and n[1]not in[_F,'o','b','x']:return _A
	if n[0]==_Q or n[0]==_O:
		n=n[1:]
		if len(n)>1 and n[0]=='0'and n[1]!=_F:return _A
	if'__'in n:return _A
	return _B
def load(f,_dict=dict,decoder=_E):
	B=_dict;A=decoder
	if _ispath(f):
		with io.open(_getpath(f),encoding='utf-8')as G:return loads(G.read(),B,A)
	elif isinstance(f,list):
		from os import path as D;from warnings import warn
		if not[A for A in f if D.exists(A)]:C='Load expects a list to contain filenames only.';C+=linesep;C+='The list needs to contain the path of at least one existing file.';raise FNFError(C)
		if A is _E:A=TomlDecoder(B)
		E=A.get_empty_table()
		for F in f:
			if D.exists(F):E.update(load(F,B,A))
			else:warn('Non-existent filename in list with at least one valid filename')
		return E
	else:
		try:return loads(f.read(),B,A)
		except AttributeError:raise TypeError('You can only load a file descriptor, filename or list')
_groupname_re=re.compile('^[A-Za-z0-9_-]+$')
def loads(s,_dict=dict,decoder=_E):
	q="Invalid group name '";K=decoder;d=[]
	if K is _E:K=TomlDecoder(_dict)
	e=K.get_empty_table();G=e
	if not isinstance(s,basestring):raise TypeError('Expecting something like a string')
	if not isinstance(s,unicode):s=s.decode('utf8')
	I=s;B=list(s);b=0;J=_A;Q='';D=_A;L=_A;V=_B;U=_A;W=_A;O=0;c='';j='';k=1
	for (A,E) in enumerate(B):
		if E=='\r'and B[A+1]==_G:B[A]=_L;continue
		if O:
			c+=E
			if E==_G:raise TomlDecodeError('Key name found without value. Reached end of line.',I,A)
			if J:
				if E==Q:
					S=_A;F=1
					while A>=F and B[A-F]==_H:S=not S;F+=1
					if not S:O=2;J=_A;Q=''
				continue
			elif O==1:
				if E.isspace():O=2;continue
				elif E==_F:W=_B;continue
				elif E.isalnum()or E==_P or E==_O:continue
				elif W and B[A-1]==_F and(E==_C or E==_D):J=_B;Q=E;continue
			elif O==2:
				if E.isspace():
					if W:
						X=B[A+1]
						if not X.isspace()and X!=_F:O=1
					continue
				if E==_F:
					W=_B;X=B[A+1]
					if not X.isspace()and X!=_F:O=1
					continue
			if E==_J:O=0;j=c[:-1].rstrip();c='';W=_A
			else:raise TomlDecodeError("Found invalid character in key name: '"+E+"'. Try quoting the key name.",I,A)
		if E==_D and Q!=_C:
			F=1
			try:
				while B[A-F]==_D:
					F+=1
					if F==3:break
			except IndexError:pass
			if F==3:D=not D;J=D
			else:J=not J
			if J:Q=_D
			else:Q=''
		if E==_C and Q!=_D:
			S=_A;F=1;f=_A
			try:
				while B[A-F]==_C:
					F+=1
					if F==3:f=_B;break
				if F==1 or F==3 and f:
					while B[A-F]==_H:S=not S;F+=1
			except IndexError:pass
			if not S:
				if f:D=not D;J=D
				else:J=not J
			if J:Q=_C
			else:Q=''
		if E=='#'and(not J and not U and not L):
			R=A;l=''
			try:
				while B[R]!=_G:l+=s[R];B[R]=_L;R+=1
			except IndexError:break
			if not b:K.preserve_comment(k,j,l,V)
		if E==_I and(not J and not U and not L):
			if V:
				if len(B)>A+1 and B[A+1]==_I:L=_B
				else:U=_B
			else:b+=1
		if E==_M and not J:
			if U:U=_A
			elif L:
				if B[A-1]==_M:L=_A
			else:b-=1
		if E==_G:
			if J or D:
				if not D:raise TomlDecodeError('Unbalanced quotes',I,A)
				if(B[A-1]==_D or B[A-1]==_C)and B[A-2]==B[A-1]:
					B[A]=B[A-1]
					if B[A-3]==B[A-1]:B[A-3]=_L
			elif b:B[A]=_L
			else:V=_B
			k+=1
		elif V and B[A]!=_L and B[A]!=_S:
			V=_A
			if not U and not L:
				if B[A]==_J:raise TomlDecodeError('Found empty keyname. ',I,A)
				O=1;c+=E
	if O:raise TomlDecodeError('Key name found without value. Reached end of file.',I,len(s))
	if J:raise TomlDecodeError('Unterminated string found. Reached end of file.',I,len(s))
	s=''.join(B);s=s.split(_G);T=_E;D='';P=_A;N=0
	for (g,C) in enumerate(s):
		if g>0:N+=len(s[g-1])+1
		K.embed_comments(g,G)
		if not D or P or _G not in D:C=C.strip()
		if C==''and(not T or P):continue
		if T:
			if P:D+=C
			else:D+=C
			P=_A;h=_A
			if D[0]==_I:h=C[-1]==_M
			elif len(C)>2:h=C[-1]==D[0]and C[-2]==D[0]and C[-3]==D[0]
			if h:
				try:o,r=K.load_value(D)
				except ValueError as Y:raise TomlDecodeError(str(Y),I,N)
				G[T]=o;T=_E;D=''
			else:
				F=len(D)-1
				while F>-1 and D[F]==_H:P=not P;F-=1
				if P:D=D[:-1]
				else:D+=_G
			continue
		if C[0]==_I:
			L=_A
			if len(C)==1:raise TomlDecodeError('Opening key group bracket on line by itself.',I,N)
			if C[1]==_I:L=_B;C=C[2:];Z=']]'
			else:C=C[1:];Z=_M
			A=1;p=K._get_split_on_quotes(C);i=_A
			for m in p:
				if not i and Z in m:break
				A+=m.count(Z);i=not i
			C=C.split(Z,A)
			if len(C)<A+1 or C[-1].strip()!='':raise TomlDecodeError('Key group not on a line by itself.',I,N)
			H=Z.join(C[:-1]).split(_F);A=0
			while A<len(H):
				H[A]=H[A].strip()
				if len(H[A])>0 and(H[A][0]==_C or H[A][0]==_D):
					a=H[A];R=A+1
					while not a[0]==a[-1]:
						R+=1
						if R>len(H)+2:raise TomlDecodeError(q+a+"' Something "+'went wrong.',I,N)
						a=_F.join(H[A:R]).strip()
					H[A]=a[1:-1];H[A+1:R]=[]
				elif not _groupname_re.match(H[A]):raise TomlDecodeError(q+H[A]+"'. Try quoting it.",I,N)
				A+=1
			G=e
			for A in _range(len(H)):
				M=H[A]
				if M=='':raise TomlDecodeError("Can't have a keygroup with an empty name",I,N)
				try:
					G[M]
					if A==len(H)-1:
						if M in d:
							d.remove(M)
							if L:raise TomlDecodeError("An implicitly defined table can't be an array",I,N)
						elif L:G[M].append(K.get_empty_table())
						else:raise TomlDecodeError('What? '+M+' already exists?'+str(G),I,N)
				except TypeError:
					G=G[-1]
					if M not in G:
						G[M]=K.get_empty_table()
						if A==len(H)-1 and L:G[M]=[K.get_empty_table()]
				except KeyError:
					if A!=len(H)-1:d.append(M)
					G[M]=K.get_empty_table()
					if A==len(H)-1 and L:G[M]=[K.get_empty_table()]
				G=G[M]
				if L:
					try:G=G[-1]
					except KeyError:pass
		elif C[0]==_K:
			if C[-1]!=_R:raise TomlDecodeError('Line breaks are not allowed in inlineobjects',I,N)
			try:K.load_inline_object(C,G,T,P)
			except ValueError as Y:raise TomlDecodeError(str(Y),I,N)
		elif _J in C:
			try:n=K.load_line(C,G,T,P)
			except ValueError as Y:raise TomlDecodeError(str(Y),I,N)
			if n is not _E:T,D,P=n
	return e
def _load_date(val):
	I='Z';A=val;G=0;F=_E
	try:
		if len(A)>19:
			if A[19]==_F:
				if A[-1].upper()==I:C=A[20:-1];D=I
				else:
					B=A[20:]
					if _Q in B:E=B.index(_Q);C=B[:E];D=B[E:]
					elif _O in B:E=B.index(_O);C=B[:E];D=B[E:]
					else:D=_E;C=B
				if D is not _E:F=TomlTz(D)
				G=int(int(C)*10**(6-len(C)))
			else:F=TomlTz(A[19:])
	except ValueError:F=_E
	if _O not in A[1:]:return _E
	try:
		if len(A)==10:H=datetime.date(int(A[:4]),int(A[5:7]),int(A[8:10]))
		else:H=datetime.datetime(int(A[:4]),int(A[5:7]),int(A[8:10]),int(A[11:13]),int(A[14:16]),int(A[17:19]),G,F)
	except ValueError:return _E
	return H
def _load_unicode_escapes(v,hexbytes,prefix):
	G='Invalid escape sequence: ';E=prefix;C=_A;A=len(v)-1
	while A>-1 and v[A]==_H:C=not C;A-=1
	for D in hexbytes:
		if C:
			C=_A;A=len(D)-1
			while A>-1 and D[A]==_H:C=not C;A-=1
			v+=E;v+=D;continue
		B='';A=0;F=4
		if E==_V:F=8
		B=''.join(D[A:A+F]).lower()
		if B.strip('0123456789abcdef'):raise ValueError(G+B)
		if B[0]=='d'and B[1].strip('01234567'):raise ValueError(G+B+'. Only scalar unicode points are allowed.')
		v+=unichr(int(B,16));v+=unicode(D[len(B):])
	return v
_escapes=['0','b','f','n','r','t',_C]
_escapedchars=['\x00','\x08','\x0c',_G,'\r',_S,_C]
_escape_to_escapedchars=dict(zip(_escapes,_escapedchars))
def _unescape(v):
	A=0;B=_A
	while A<len(v):
		if B:
			B=_A
			if v[A]in _escapes:v=v[:A-1]+_escape_to_escapedchars[v[A]]+v[A+1:]
			elif v[A]==_H:v=v[:A-1]+v[A:]
			elif v[A]=='u'or v[A]=='U':A+=1
			else:raise ValueError(_W)
			continue
		elif v[A]==_H:B=_B
		A+=1
	return v
class InlineTableDict:0
class TomlDecoder:
	def __init__(A,_dict=dict):A._dict=_dict
	def get_empty_table(A):return A._dict()
	def get_empty_inline_table(A):
		class B(A._dict,InlineTableDict):0
		return B()
	def load_inline_object(E,line,currentlevel,multikey=_A,multibackslash=_A):
		B=line[1:-1].split(_N);D=[]
		if len(B)==1 and not B[0].strip():B.pop()
		while len(B)>0:
			C=B.pop(0)
			try:H,A=C.split(_J,1)
			except ValueError:raise ValueError('Invalid inline table encountered')
			A=A.strip()
			if A[0]==A[-1]and A[0]in(_C,_D)or(A[0]in'-0123456789'or A in(_T,_U)or A[0]==_I and A[-1]==_M or A[0]==_K and A[-1]==_R):D.append(C)
			elif len(B)>0:B[0]=C+_N+B[0]
			else:raise ValueError('Invalid inline table value encountered')
		for F in D:
			G=E.load_line(F,currentlevel,multikey,multibackslash)
			if G is not _E:break
	def _get_split_on_quotes(F,line):
		A=line.split(_C);D=_A;C=[]
		if len(A)>1 and _D in A[0]:
			B=A[0].split(_D);A=A[1:]
			while len(B)%2==0 and len(A):
				B[-1]+=_C+A[0];A=A[1:]
				if _D in B[-1]:B=B[:-1]+B[-1].split(_D)
			C+=B
		for E in A:
			if D:C.append(E)
			else:C+=E.split(_D);D=not D
		return C
	def load_line(E,line,currentlevel,multikey,multibackslash):
		S='Duplicate keys!';L=multikey;K=line;G=multibackslash;D=currentlevel;H=1;M=E._get_split_on_quotes(K);C=_A
		for F in M:
			if not C and _J in F:break
			H+=F.count(_J);C=not C
		A=K.split(_J,H);N=_strictly_valid_num(A[-1])
		if _number_with_underscores.match(A[-1]):A[-1]=A[-1].replace(_P,'')
		while len(A[-1])and(A[-1][0]!=_L and A[-1][0]!=_S and A[-1][0]!=_D and A[-1][0]!=_C and A[-1][0]!=_I and A[-1][0]!=_K and A[-1].strip()!=_T and A[-1].strip()!=_U):
			try:float(A[-1]);break
			except ValueError:pass
			if _load_date(A[-1])is not _E:break
			if TIME_RE.match(A[-1]):break
			H+=1;P=A[-1];A=K.split(_J,H)
			if P==A[-1]:raise ValueError('Invalid date or number')
			if N:N=_strictly_valid_num(A[-1])
		A=[_J.join(A[:-1]).strip(),A[-1].strip()]
		if _F in A[0]:
			if _C in A[0]or _D in A[0]:
				M=E._get_split_on_quotes(A[0]);C=_A;B=[]
				for F in M:
					if C:B.append(F)
					else:B+=[A.strip()for A in F.split(_F)]
					C=not C
			else:B=A[0].split(_F)
			while B[-1]=='':B=B[:-1]
			for I in B[:-1]:
				if I=='':continue
				if I not in D:D[I]=E.get_empty_table()
				D=D[I]
			A[0]=B[-1].strip()
		elif(A[0][0]==_C or A[0][0]==_D)and A[0][-1]==A[0][0]:A[0]=_unescape(A[0][1:-1])
		J,Q=E._load_line_multiline_str(A[1])
		if J>-1:
			while J>-1 and A[1][J+Q]==_H:G=not G;J-=1
			if G:O=A[1][:-1]
			else:O=A[1]+_G
			L=A[0]
		else:R,T=E.load_value(A[1],N)
		try:D[A[0]];raise ValueError(S)
		except TypeError:raise ValueError(S)
		except KeyError:
			if L:return L,O,G
			else:D[A[0]]=R
	def _load_line_multiline_str(C,p):
		B=0
		if len(p)<3:return-1,B
		if p[0]==_I and(p.strip()[-1]!=_M and C._load_array_isstrarray(p)):
			A=p[1:].strip().split(_N)
			while len(A)>1 and A[-1][0]!=_C and A[-1][0]!=_D:A=A[:-2]+[A[-2]+_N+A[-1]]
			A=A[-1];B=len(p)-len(A);p=A
		if p[0]!=_C and p[0]!=_D:return-1,B
		if p[1]!=p[0]or p[2]!=p[0]:return-1,B
		if len(p)>5 and p[-1]==p[0]and p[-2]==p[0]and p[-3]==p[0]:return-1,B
		return len(p)-1,B
	def load_value(E,v,strictly_valid=_B):
		a='float';Z='int';Y='bool'
		if not v:raise ValueError('Empty value is invalid')
		if v==_T:return _B,Y
		elif v==_U:return _A,Y
		elif v[0]==_C or v[0]==_D:
			F=v[0];B=v[1:].split(F);G=_A;H=0
			if len(B)>1 and B[0]==''and B[1]=='':B=B[2:];G=_B
			I=_A
			for J in B:
				if J=='':
					if G:H+=1
					else:I=_B
				else:
					K=_A
					try:
						A=-1;N=J[A]
						while N==_H:K=not K;A-=1;N=J[A]
					except IndexError:pass
					if not K:
						if I:raise ValueError('Found tokens after a closed '+'string. Invalid TOML.')
						elif not G or H>1:I=_B
						else:H=0
			if F==_C:
				T=v.split(_H)[1:];C=_A
				for A in T:
					if A=='':C=not C
					else:
						if A[0]not in _escapes and(A[0]!='u'and A[0]!='U'and not C):raise ValueError(_W)
						if C:C=_A
				for L in ['\\u',_V]:
					if L in v:O=v.split(L);v=_load_unicode_escapes(O[0],O[1:],L)
				v=_unescape(v)
			if len(v)>1 and v[1]==F and(len(v)<3 or v[1]==v[2]):v=v[2:-2]
			return v[1:-1],'str'
		elif v[0]==_I:return E.load_array(v),'array'
		elif v[0]==_K:P=E.get_empty_inline_table();E.load_inline_object(v,P);return P,'inline_object'
		elif TIME_RE.match(v):U,V,W,b,Q=TIME_RE.match(v).groups();X=datetime.time(int(U),int(V),int(W),int(Q)if Q else 0);return X,'time'
		else:
			R=_load_date(v)
			if R is not _E:return R,'date'
			if not strictly_valid:raise ValueError('Weirdness with leading zeroes or underscores in your number.')
			D=Z;S=_A
			if v[0]==_O:S=_B;v=v[1:]
			elif v[0]==_Q:v=v[1:]
			v=v.replace(_P,'');M=v.lower()
			if _F in v or'x'not in v and('e'in v or'E'in v):
				if _F in v and v.split(_F,1)[1]=='':raise ValueError('This float is missing digits after the point')
				if v[0]not in'0123456789':raise ValueError("This float doesn't have a leading digit")
				v=float(v);D=a
			elif len(M)==3 and(M=='inf'or M=='nan'):v=float(v);D=a
			if D==Z:v=int(v,0)
			if S:return 0-v,D
			return v,D
	def bounded_string(C,s):
		if len(s)==0:return _B
		if s[-1]!=s[0]:return _A
		A=-2;B=_A
		while len(s)+A>0:
			if s[A]==_H:B=not B;A-=1
			else:break
		return not B
	def _load_array_isstrarray(A,a):
		a=a[1:-1].strip()
		if a!=''and(a[0]==_C or a[0]==_D):return _B
		return _A
	def load_array(H,a):
		I=_E;N=[];a=a.strip()
		if _I not in a[1:-1]or''!=a[1:-1].split(_I)[0].strip():
			Q=H._load_array_isstrarray(a)
			if not a[1:-1].strip().startswith(_K):a=a[1:-1].split(_N)
			else:
				O=[];E=1;A=2;J=1 if a[E]==_K else 0;F=_A
				while A<len(a[1:]):
					if a[A]==_C or a[A]==_D:
						if F:
							K=A-1
							while K>-1 and a[K]==_H:F=not F;K-=1
						F=not F
					if not F and a[A]==_K:J+=1
					if F or a[A]!=_R:A+=1;continue
					elif a[A]==_R and J>1:J-=1;A+=1;continue
					A+=1;O.append(a[E:A]);E=A+1
					while E<len(a[1:])and a[E]!=_K:E+=1
					A=E+1
				a=O
			B=0
			if Q:
				while B<len(a)-1:
					C=a[B].strip()
					while not H.bounded_string(C)or len(C)>2 and C[0]==C[1]==C[2]and C[-2]!=C[0]and C[-3]!=C[0]:
						a[B]=a[B]+_N+a[B+1];C=a[B].strip()
						if B<len(a)-2:a=a[:B+1]+a[B+2:]
						else:a=a[:B+1]
					B+=1
		else:
			G=list(a[1:-1]);a=[];L=0;M=0
			for D in _range(len(G)):
				if G[D]==_I:L+=1
				elif G[D]==_M:L-=1
				elif G[D]==_N and not L:a.append(''.join(G[M:D]));M=D+1
			a.append(''.join(G[M:]))
		for D in _range(len(a)):
			a[D]=a[D].strip()
			if a[D]!='':
				R,P=H.load_value(a[D])
				if I:
					if P!=I:raise ValueError('Not a homogeneous array')
				else:I=P
				N.append(R)
		return N
	def preserve_comment(A,line_no,key,comment,beginline):0
	def embed_comments(A,idx,currentlevel):0
class TomlPreserveCommentDecoder(TomlDecoder):
	def __init__(A,_dict=dict):A.saved_comments={};super(TomlPreserveCommentDecoder,A).__init__(_dict)
	def preserve_comment(A,line_no,key,comment,beginline):A.saved_comments[line_no]=key,comment,beginline
	def embed_comments(A,idx,currentlevel):
		B=currentlevel
		if idx not in A.saved_comments:return
		C,D,E=A.saved_comments[idx];B[C]=CommentValue(B[C],D,E,A._dict)


"""Dynaconf django extension

In the `django_project/settings.py` put at the very botton of the file:

# HERE STARTS DYNACONF EXTENSION LOAD (Keep at the very bottom of settings.py)
# Read more at https://dynaconf.readthedocs.io/en/latest/guides/django.html
import dynaconf  # noqa
settings = dynaconf.DjangoDynaconf(__name__)  # noqa
# HERE ENDS DYNACONF EXTENSION LOAD (No more code below this line)

Now in the root of your Django project
(the same folder where manage.py is located)

Put your config files `settings.{py|yaml|toml|ini|json}`
and or `.secrets.{py|yaml|toml|ini|json}`

On your projects root folder now you can start as::

    DJANGO_DEBUG='false' \
    DJANGO_ALLOWED_HOSTS='["localhost"]' \
    python manage.py runserver
"""
import inspect
import os
import sys

import dynaconf

try:  # pragma: no cover
    from django import conf
    from django.conf import settings as django_settings

    django_installed = True
except ImportError:  # pragma: no cover
    django_installed = False


def load(django_settings_module_name=None, **kwargs):  # pragma: no cover
    if not django_installed:
        raise RuntimeError(
            "To use this extension django must be installed "
            "install it with: pip install django"
        )

    try:
        django_settings_module = sys.modules[django_settings_module_name]
    except KeyError:
        django_settings_module = sys.modules[
            os.environ["DJANGO_SETTINGS_MODULE"]
        ]

    settings_file = os.path.abspath(django_settings_module.__file__)
    _root_path = os.path.dirname(settings_file)

    # 1) Create the lazy settings object reusing settings_module consts
    options = {
        k: v for k, v in django_settings_module.__dict__.items() if k.isupper()
    }
    options.update(kwargs)
    options.setdefault(
        "SKIP_FILES_FOR_DYNACONF", [settings_file, "dynaconf_merge"]
    )
    options.setdefault("ROOT_PATH_FOR_DYNACONF", _root_path)
    options.setdefault("ENVVAR_PREFIX_FOR_DYNACONF", "DJANGO")
    options.setdefault("ENV_SWITCHER_FOR_DYNACONF", "DJANGO_ENV")
    options.setdefault("ENVIRONMENTS_FOR_DYNACONF", True)
    options.setdefault("load_dotenv", True)
    options.setdefault(
        "default_settings_paths", dynaconf.DEFAULT_SETTINGS_FILES
    )

    lazy_settings = dynaconf.LazySettings(**options)
    dynaconf.settings = lazy_settings  # rebind the settings

    # 2) Set all settings back to django_settings_module for 'django check'
    lazy_settings.populate_obj(django_settings_module)

    # 3) Bind `settings` and `DYNACONF`
    setattr(django_settings_module, "settings", lazy_settings)
    setattr(django_settings_module, "DYNACONF", lazy_settings)

    # 4) keep django original settings
    dj = {}
    for key in dir(django_settings):
        if (
            key.isupper()
            and (key != "SETTINGS_MODULE")
            and key not in lazy_settings.store
        ):
            dj[key] = getattr(django_settings, key, None)
        dj["ORIGINAL_SETTINGS_MODULE"] = django_settings.SETTINGS_MODULE

    lazy_settings.update(dj)

    # 5) Patch django.conf.settings
    class Wrapper(object):

        # lazy_settings = conf.settings.lazy_settings

        def __getattribute__(self, name):
            if name == "settings":
                return lazy_settings
            else:
                return getattr(conf, name)

    # This implementation is recommended by Guido Van Rossum
    # https://mail.python.org/pipermail/python-ideas/2012-May/014969.html
    sys.modules["django.conf"] = Wrapper()

    # 6) Enable standalone scripts to use Dynaconf
    # This is for when `django.conf.settings` is imported directly
    # on external `scripts` (out of Django's lifetime)
    for stack_item in reversed(inspect.stack()):
        if isinstance(
            stack_item.frame.f_globals.get("settings"), conf.LazySettings
        ):
            stack_item.frame.f_globals["settings"] = lazy_settings

    return lazy_settings


# syntax sugar
DjangoDynaconf = load  # noqa



_D='strict'
_C=True
_B=False
_A=None
import os,sys
from ._compat import _default_text_stderr,_default_text_stdout,_find_binary_writer,auto_wrap_for_ansi,binary_streams,filename_to_ui,get_filesystem_encoding,get_strerror,is_bytes,open_stream,should_strip_ansi,strip_ansi,text_streams,WIN
from .globals import resolve_color_default
echo_native_types=str,bytes,bytearray
def _posixify(name):return '-'.join(name.split()).lower()
def safecall(func):
	def A(*A,**B):
		try:return func(*A,**B)
		except Exception:pass
	return A
def make_str(value):
	A=value
	if isinstance(A,bytes):
		try:return A.decode(get_filesystem_encoding())
		except UnicodeError:return A.decode('utf-8','replace')
	return str(A)
def make_default_short_help(help,max_length=45):
	F=help.split();D=0;A=[];C=_B
	for B in F:
		if B[-1:]=='.':C=_C
		E=1+len(B)if A else len(B)
		if D+E>max_length:A.append('...');C=_C
		else:
			if A:A.append(' ')
			A.append(B)
		if C:break
		D+=E
	return ''.join(A)
class LazyFile:
	def __init__(A,filename,mode='r',encoding=_A,errors=_D,atomic=_B):
		E=errors;D=encoding;C=mode;B=filename;A.name=B;A.mode=C;A.encoding=D;A.errors=E;A.atomic=atomic
		if B=='-':A._f,A.should_close=open_stream(B,C,D,E)
		else:
			if'r'in C:open(B,C).close()
			A._f=_A;A.should_close=_C
	def __getattr__(A,name):return getattr(A.open(),name)
	def __repr__(A):
		if A._f is not _A:return repr(A._f)
		return f"<unopened file '{A.name}' {A.mode}>"
	def open(A):
		if A._f is not _A:return A._f
		try:B,A.should_close=open_stream(A.name,A.mode,A.encoding,A.errors,atomic=A.atomic)
		except OSError as C:from .exceptions import FileError as D;raise D(A.name,hint=get_strerror(C))
		A._f=B;return B
	def close(A):
		if A._f is not _A:A._f.close()
	def close_intelligently(A):
		if A.should_close:A.close()
	def __enter__(A):return A
	def __exit__(A,exc_type,exc_value,tb):A.close_intelligently()
	def __iter__(A):A.open();return iter(A._f)
class KeepOpenFile:
	def __init__(A,file):A._file=file
	def __getattr__(A,name):return getattr(A._file,name)
	def __enter__(A):return A
	def __exit__(A,exc_type,exc_value,tb):0
	def __repr__(A):return repr(A._file)
	def __iter__(A):return iter(A._file)
def echo(message=_A,file=_A,nl=_C,err=_B,color=_A):
	C=color;B=file;A=message
	if B is _A:
		if err:B=_default_text_stderr()
		else:B=_default_text_stdout()
	if A is not _A and not isinstance(A,echo_native_types):A=str(A)
	if nl:
		A=A or''
		if isinstance(A,str):A+='\n'
		else:A+=b'\n'
	if A and is_bytes(A):
		D=_find_binary_writer(B)
		if D is not _A:B.flush();D.write(A);D.flush();return
	if A and not is_bytes(A):
		C=resolve_color_default(C)
		if should_strip_ansi(B,C):A=strip_ansi(A)
		elif WIN:
			if auto_wrap_for_ansi is not _A:B=auto_wrap_for_ansi(B)
			elif not C:A=strip_ansi(A)
	if A:B.write(A)
	B.flush()
def get_binary_stream(name):
	A=binary_streams.get(name)
	if A is _A:raise TypeError(f"Unknown standard stream '{name}'")
	return A()
def get_text_stream(name,encoding=_A,errors=_D):
	A=text_streams.get(name)
	if A is _A:raise TypeError(f"Unknown standard stream '{name}'")
	return A(encoding,errors)
def open_file(filename,mode='r',encoding=_A,errors=_D,lazy=_B,atomic=_B):
	E=atomic;D=errors;C=encoding;B=filename
	if lazy:return LazyFile(B,mode,C,D,atomic=E)
	A,F=open_stream(B,mode,C,D,atomic=E)
	if not F:A=KeepOpenFile(A)
	return A
def get_os_args():import warnings as A;A.warn("'get_os_args' is deprecated and will be removed in 8.1. Access 'sys.argv[1:]' directly instead.",DeprecationWarning,stacklevel=2);return sys.argv[1:]
def format_filename(filename,shorten=_B):
	A=filename
	if shorten:A=os.path.basename(A)
	return filename_to_ui(A)
def get_app_dir(app_name,roaming=_C,force_posix=_B):
	A=app_name
	if WIN:
		C='APPDATA'if roaming else'LOCALAPPDATA';B=os.environ.get(C)
		if B is _A:B=os.path.expanduser('~')
		return os.path.join(B,A)
	if force_posix:return os.path.join(os.path.expanduser(f"~/.{_posixify(A)}"))
	if sys.platform=='darwin':return os.path.join(os.path.expanduser('~/Library/Application Support'),A)
	return os.path.join(os.environ.get('XDG_CONFIG_HOME',os.path.expanduser('~/.config')),_posixify(A))
class PacifyFlushWrapper:
	def __init__(A,wrapped):A.wrapped=wrapped
	def flush(A):
		try:A.wrapped.flush()
		except OSError as B:
			import errno
			if B.errno!=errno.EPIPE:raise
	def __getattr__(A,attr):return getattr(A.wrapped,attr)